# Copyright (C) 2021 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Data for for builtin setting classes."""

from ansys.fluent.core.utils.fluent_version import FluentVersion

# {<class name>: (<kind>, <path>)}
DATA = {
    "Setup": ("Singleton", "setup"),
    "General": ("Singleton", "setup.general"),
    "Models": ("Singleton", "setup.models"),
    "Multiphase": ("Singleton", "setup.models.multiphase"),
    "Energy": ("Singleton", "setup.models.energy"),
    "Viscous": ("Singleton", "setup.models.viscous"),
    "Radiation": (
        "Singleton",
        {
            FluentVersion.v252: "setup.models.radiation",
            FluentVersion.v251: "setup.models.radiation",
            FluentVersion.v242: "setup.models.radiation",
            FluentVersion.v241: "setup.models.radiation",
            FluentVersion.v232: "setup.models.radiation",
        },
    ),
    "Species": (
        "Singleton",
        {
            FluentVersion.v252: "setup.models.species",
            FluentVersion.v251: "setup.models.species",
            FluentVersion.v242: "setup.models.species",
            FluentVersion.v241: "setup.models.species",
            FluentVersion.v232: "setup.models.species",
        },
    ),
    "DiscretePhase": (
        "Singleton",
        {
            FluentVersion.v252: "setup.models.discrete_phase",
            FluentVersion.v251: "setup.models.discrete_phase",
            FluentVersion.v242: "setup.models.discrete_phase",
            FluentVersion.v241: "setup.models.discrete_phase",
            FluentVersion.v232: "setup.models.discrete_phase",
            FluentVersion.v231: "setup.models.discrete_phase",
        },
    ),
    "Injections": (
        "Singleton",
        {
            FluentVersion.v252: "setup.models.discrete_phase.injections",
            FluentVersion.v251: "setup.models.discrete_phase.injections",
            FluentVersion.v242: "setup.models.discrete_phase.injections",
            FluentVersion.v241: "setup.models.discrete_phase.injections",
            FluentVersion.v232: "setup.models.discrete_phase.injections",
            FluentVersion.v231: "setup.models.discrete_phase.injections",
        },
    ),
    "Injection": (
        "NamedObject",
        {
            FluentVersion.v252: "setup.models.discrete_phase.injections",
            FluentVersion.v251: "setup.models.discrete_phase.injections",
            FluentVersion.v242: "setup.models.discrete_phase.injections",
            FluentVersion.v241: "setup.models.discrete_phase.injections",
            FluentVersion.v232: "setup.models.discrete_phase.injections",
            FluentVersion.v231: "setup.models.discrete_phase.injections",
        },
    ),
    "VirtualBladeModel": (
        "Singleton",
        {
            FluentVersion.v252: "setup.models.virtual_blade_model",
            FluentVersion.v251: "setup.models.virtual_blade_model",
            FluentVersion.v242: "setup.models.virtual_blade_model",
            FluentVersion.v241: "setup.models.virtual_blade_model",
            FluentVersion.v232: "setup.models.virtual_blade_model",
            FluentVersion.v231: "setup.models.virtual_blade_model",
        },
    ),
    "Optics": (
        "Singleton",
        {
            FluentVersion.v252: "setup.models.optics",
            FluentVersion.v251: "setup.models.optics",
            FluentVersion.v242: "setup.models.optics",
            FluentVersion.v241: "setup.models.optics",
            FluentVersion.v232: "setup.models.optics",
            FluentVersion.v231: "setup.models.optics",
        },
    ),
    "Structure": (
        "Singleton",
        {
            FluentVersion.v252: "setup.models.structure",
            FluentVersion.v251: "setup.models.structure",
            FluentVersion.v242: "setup.models.structure",
            FluentVersion.v241: "setup.models.structure",
            FluentVersion.v232: "setup.models.structure",
        },
    ),
    "Ablation": (
        "Singleton",
        {
            FluentVersion.v252: "setup.models.ablation",
            FluentVersion.v251: "setup.models.ablation",
            FluentVersion.v242: "setup.models.ablation",
            FluentVersion.v241: "setup.models.ablation",
            FluentVersion.v232: "setup.models.ablation",
        },
    ),
    "EChemistry": (
        "Singleton",
        {
            FluentVersion.v252: "setup.models.echemistry",
            FluentVersion.v251: "setup.models.echemistry",
            FluentVersion.v242: "setup.models.echemistry",
            FluentVersion.v241: "setup.models.echemistry",
        },
    ),
    "Battery": (
        "Singleton",
        {
            FluentVersion.v252: "setup.models.battery",
            FluentVersion.v251: "setup.models.battery",
            FluentVersion.v242: "setup.models.battery",
            FluentVersion.v241: "setup.models.battery",
        },
    ),
    "SystemCoupling": (
        "Singleton",
        {
            FluentVersion.v252: "setup.models.system_coupling",
            FluentVersion.v251: "setup.models.system_coupling",
            FluentVersion.v242: "setup.models.system_coupling",
            FluentVersion.v241: "setup.models.system_coupling",
        },
    ),
    "Sofc": (
        "Singleton",
        {
            FluentVersion.v252: "setup.models.sofc",
            FluentVersion.v251: "setup.models.sofc",
            FluentVersion.v242: "setup.models.sofc",
            FluentVersion.v241: "setup.models.sofc",
        },
    ),
    "Pemfc": (
        "Singleton",
        {
            FluentVersion.v252: "setup.models.pemfc",
            FluentVersion.v251: "setup.models.pemfc",
            FluentVersion.v242: "setup.models.pemfc",
        },
    ),
    "Materials": ("Singleton", "setup.materials"),
    "FluidMaterials": ("Singleton", "setup.materials.fluid"),
    "FluidMaterial": ("NamedObject", "setup.materials.fluid"),
    "SolidMaterials": ("Singleton", "setup.materials.solid"),
    "SolidMaterial": ("NamedObject", "setup.materials.solid"),
    "MixtureMaterials": ("Singleton", "setup.materials.mixture"),
    "MixtureMaterial": ("NamedObject", "setup.materials.mixture"),
    "ParticleMixtureMaterials": ("Singleton", "setup.materials.particle_mixture"),
    "ParticleMixtureMaterial": ("NamedObject", "setup.materials.particle_mixture"),
    "CellZoneConditions": ("Singleton", "setup.cell_zone_conditions"),
    "CellZoneCondition": (
        "NamedObject",
        {
            FluentVersion.v252: "setup.cell_zone_conditions",
            FluentVersion.v251: "setup.cell_zone_conditions",
            FluentVersion.v242: "setup.cell_zone_conditions",
            FluentVersion.v241: "setup.cell_zone_conditions",
            FluentVersion.v232: "setup.cell_zone_conditions",
            FluentVersion.v231: "setup.cell_zone_conditions",
        },
    ),
    "FluidCellZones": ("Singleton", "setup.cell_zone_conditions.fluid"),
    "FluidCellZone": ("NamedObject", "setup.cell_zone_conditions.fluid"),
    "SolidCellZones": ("Singleton", "setup.cell_zone_conditions.solid"),
    "SolidCellZone": ("NamedObject", "setup.cell_zone_conditions.solid"),
    "BoundaryConditions": ("Singleton", "setup.boundary_conditions"),
    "BoundaryCondition": (
        "NamedObject",
        {
            FluentVersion.v252: "setup.boundary_conditions",
            FluentVersion.v251: "setup.boundary_conditions",
            FluentVersion.v242: "setup.boundary_conditions",
            FluentVersion.v241: "setup.boundary_conditions",
            FluentVersion.v232: "setup.boundary_conditions",
            FluentVersion.v231: "setup.boundary_conditions",
        },
    ),
    "AxisBoundaries": ("Singleton", "setup.boundary_conditions.axis"),
    "AxisBoundary": ("NamedObject", "setup.boundary_conditions.axis"),
    "DegassingBoundaries": ("Singleton", "setup.boundary_conditions.degassing"),
    "DegassingBoundary": ("NamedObject", "setup.boundary_conditions.degassing"),
    "ExhaustFanBoundaries": ("Singleton", "setup.boundary_conditions.exhaust_fan"),
    "ExhaustFanBoundary": ("NamedObject", "setup.boundary_conditions.exhaust_fan"),
    "FanBoundaries": ("Singleton", "setup.boundary_conditions.fan"),
    "FanBoundary": ("NamedObject", "setup.boundary_conditions.fan"),
    "GeometryBoundaries": ("Singleton", "setup.boundary_conditions.geometry"),
    "GeometryBoundary": ("NamedObject", "setup.boundary_conditions.geometry"),
    "InletVentBoundaries": ("Singleton", "setup.boundary_conditions.inlet_vent"),
    "InletVentBoundary": ("NamedObject", "setup.boundary_conditions.inlet_vent"),
    "IntakeFanBoundaries": ("Singleton", "setup.boundary_conditions.intake_fan"),
    "IntakeFanBoundary": ("NamedObject", "setup.boundary_conditions.intake_fan"),
    "InterfaceBoundaries": ("Singleton", "setup.boundary_conditions.interface"),
    "InterfaceBoundary": ("NamedObject", "setup.boundary_conditions.interface"),
    "InteriorBoundaries": ("Singleton", "setup.boundary_conditions.interior"),
    "InteriorBoundary": ("NamedObject", "setup.boundary_conditions.interior"),
    "MassFlowInlets": ("Singleton", "setup.boundary_conditions.mass_flow_inlet"),
    "MassFlowInlet": ("NamedObject", "setup.boundary_conditions.mass_flow_inlet"),
    "MassFlowOutlets": ("Singleton", "setup.boundary_conditions.mass_flow_outlet"),
    "MassFlowOutlet": ("NamedObject", "setup.boundary_conditions.mass_flow_outlet"),
    "NetworkBoundaries": ("Singleton", "setup.boundary_conditions.network"),
    "NetworkBoundary": ("NamedObject", "setup.boundary_conditions.network"),
    "NetworkEndBoundaries": ("Singleton", "setup.boundary_conditions.network_end"),
    "NetworkEndBoundary": ("NamedObject", "setup.boundary_conditions.network_end"),
    "OutflowBoundaries": ("Singleton", "setup.boundary_conditions.outflow"),
    "OutflowBoundary": ("NamedObject", "setup.boundary_conditions.outflow"),
    "OutletVentBoundaries": ("Singleton", "setup.boundary_conditions.outlet_vent"),
    "OutletVentBoundary": ("NamedObject", "setup.boundary_conditions.outlet_vent"),
    "OversetBoundaries": ("Singleton", "setup.boundary_conditions.overset"),
    "OversetBoundary": ("NamedObject", "setup.boundary_conditions.overset"),
    "PeriodicBoundaries": ("Singleton", "setup.boundary_conditions.periodic"),
    "PeriodicBoundary": ("NamedObject", "setup.boundary_conditions.periodic"),
    "PorousJumpBoundaries": ("Singleton", "setup.boundary_conditions.porous_jump"),
    "PorousJumpBoundary": ("NamedObject", "setup.boundary_conditions.porous_jump"),
    "PressureFarFieldBoundaries": (
        "Singleton",
        "setup.boundary_conditions.pressure_far_field",
    ),
    "PressureFarFieldBoundary": (
        "NamedObject",
        "setup.boundary_conditions.pressure_far_field",
    ),
    "PressureInlets": ("Singleton", "setup.boundary_conditions.pressure_inlet"),
    "PressureInlet": ("NamedObject", "setup.boundary_conditions.pressure_inlet"),
    "PressureOutlets": ("Singleton", "setup.boundary_conditions.pressure_outlet"),
    "PressureOutlet": ("NamedObject", "setup.boundary_conditions.pressure_outlet"),
    "RadiatorBoundaries": ("Singleton", "setup.boundary_conditions.radiator"),
    "RadiatorBoundary": ("NamedObject", "setup.boundary_conditions.radiator"),
    "RansLesInterfaceBoundaries": (
        "Singleton",
        "setup.boundary_conditions.rans_les_interface",
    ),
    "RansLesInterfaceBoundary": (
        "NamedObject",
        "setup.boundary_conditions.rans_les_interface",
    ),
    "RecirculationInlets": (
        "Singleton",
        "setup.boundary_conditions.recirculation_inlet",
    ),
    "RecirculationInlet": (
        "NamedObject",
        "setup.boundary_conditions.recirculation_inlet",
    ),
    "RecirculationOutlets": (
        "Singleton",
        "setup.boundary_conditions.recirculation_outlet",
    ),
    "RecirculationOutlet": (
        "NamedObject",
        "setup.boundary_conditions.recirculation_outlet",
    ),
    "ShadowBoundaries": ("Singleton", "setup.boundary_conditions.shadow"),
    "ShadowBoundary": ("NamedObject", "setup.boundary_conditions.shadow"),
    "SymmetryBoundaries": ("Singleton", "setup.boundary_conditions.symmetry"),
    "SymmetryBoundary": ("NamedObject", "setup.boundary_conditions.symmetry"),
    "VelocityInlets": ("Singleton", "setup.boundary_conditions.velocity_inlet"),
    "VelocityInlet": ("NamedObject", "setup.boundary_conditions.velocity_inlet"),
    "WallBoundaries": ("Singleton", "setup.boundary_conditions.wall"),
    "WallBoundary": ("NamedObject", "setup.boundary_conditions.wall"),
    "NonReflectingBoundaries": (
        "Singleton",
        {
            FluentVersion.v252: "setup.boundary_conditions.non_reflecting_bc",
            FluentVersion.v251: "setup.boundary_conditions.non_reflecting_bc",
            FluentVersion.v242: "setup.boundary_conditions.non_reflecting_bc",
            FluentVersion.v241: "setup.boundary_conditions.non_reflecting_bc",
        },
    ),
    "NonReflectingBoundary": (
        "NamedObject",
        {
            FluentVersion.v252: "setup.boundary_conditions.non_reflecting_bc",
            FluentVersion.v251: "setup.boundary_conditions.non_reflecting_bc",
            FluentVersion.v242: "setup.boundary_conditions.non_reflecting_bc",
            FluentVersion.v241: "setup.boundary_conditions.non_reflecting_bc",
        },
    ),
    "PerforatedWallBoundaries": (
        "Singleton",
        {
            FluentVersion.v252: "setup.boundary_conditions.perforated_wall",
            FluentVersion.v251: "setup.boundary_conditions.perforated_wall",
            FluentVersion.v242: "setup.boundary_conditions.perforated_wall",
            FluentVersion.v241: "setup.boundary_conditions.perforated_wall",
        },
    ),
    "PerforatedWallBoundary": (
        "NamedObject",
        {
            FluentVersion.v252: "setup.boundary_conditions.perforated_wall",
            FluentVersion.v251: "setup.boundary_conditions.perforated_wall",
            FluentVersion.v242: "setup.boundary_conditions.perforated_wall",
            FluentVersion.v241: "setup.boundary_conditions.perforated_wall",
        },
    ),
    "MeshInterfaces": (
        "Singleton",
        {
            FluentVersion.v252: "setup.mesh_interfaces",
            FluentVersion.v251: "setup.mesh_interfaces",
            FluentVersion.v242: "setup.mesh_interfaces",
            FluentVersion.v241: "setup.mesh_interfaces",
            FluentVersion.v232: "setup.mesh_interfaces",
        },
    ),
    "DynamicMesh": (
        "Singleton",
        {
            FluentVersion.v252: "setup.dynamic_mesh",
            FluentVersion.v251: "setup.dynamic_mesh",
        },
    ),
    "ReferenceValues": ("Singleton", "setup.reference_values"),
    "ReferenceFrames": (
        "Singleton",
        {
            FluentVersion.v252: "setup.reference_frames",
            FluentVersion.v251: "setup.reference_frames",
            FluentVersion.v242: "setup.reference_frames",
            FluentVersion.v241: "setup.reference_frames",
            FluentVersion.v232: "setup.reference_frames",
        },
    ),
    "ReferenceFrame": (
        "NamedObject",
        {
            FluentVersion.v252: "setup.reference_frames",
            FluentVersion.v251: "setup.reference_frames",
            FluentVersion.v242: "setup.reference_frames",
            FluentVersion.v241: "setup.reference_frames",
            FluentVersion.v232: "setup.reference_frames",
        },
    ),
    "NamedExpressions": (
        "Singleton",
        {
            FluentVersion.v252: "setup.named_expressions",
            FluentVersion.v251: "setup.named_expressions",
            FluentVersion.v242: "setup.named_expressions",
            FluentVersion.v241: "setup.named_expressions",
            FluentVersion.v232: "setup.named_expressions",
        },
    ),
    "NamedExpression": (
        "NamedObject",
        {
            FluentVersion.v252: "setup.named_expressions",
            FluentVersion.v251: "setup.named_expressions",
            FluentVersion.v242: "setup.named_expressions",
            FluentVersion.v241: "setup.named_expressions",
            FluentVersion.v232: "setup.named_expressions",
        },
    ),
    "Solution": ("Singleton", "solution"),
    "Methods": ("Singleton", "solution.methods"),
    "Controls": ("Singleton", "solution.controls"),
    "ReportDefinitions": ("Singleton", "solution.report_definitions"),
    "Monitor": (
        "Singleton",
        {
            FluentVersion.v252: "solution.monitor",
            FluentVersion.v251: "solution.monitor",
            FluentVersion.v242: "solution.monitor",
            FluentVersion.v241: "solution.monitor",
            FluentVersion.v232: "solution.monitor",
            FluentVersion.v231: "solution.monitor",
        },
    ),
    "Residual": (
        "Singleton",
        {
            FluentVersion.v252: "solution.monitor.residual",
            FluentVersion.v251: "solution.monitor.residual",
            FluentVersion.v242: "solution.monitor.residual",
            FluentVersion.v241: "solution.monitor.residual",
        },
    ),
    "ReportFiles": (
        "Singleton",
        {
            FluentVersion.v252: "solution.monitor.report_files",
            FluentVersion.v251: "solution.monitor.report_files",
            FluentVersion.v242: "solution.monitor.report_files",
            FluentVersion.v241: "solution.monitor.report_files",
            FluentVersion.v232: "solution.monitor.report_files",
            FluentVersion.v231: "solution.monitor.report_files",
        },
    ),
    "ReportFile": (
        "NamedObject",
        {
            FluentVersion.v252: "solution.monitor.report_files",
            FluentVersion.v251: "solution.monitor.report_files",
            FluentVersion.v242: "solution.monitor.report_files",
            FluentVersion.v241: "solution.monitor.report_files",
            FluentVersion.v232: "solution.monitor.report_files",
            FluentVersion.v231: "solution.monitor.report_files",
        },
    ),
    "ReportPlots": (
        "Singleton",
        {
            FluentVersion.v252: "solution.monitor.report_plots",
            FluentVersion.v251: "solution.monitor.report_plots",
            FluentVersion.v242: "solution.monitor.report_plots",
            FluentVersion.v241: "solution.monitor.report_plots",
            FluentVersion.v232: "solution.monitor.report_plots",
            FluentVersion.v231: "solution.monitor.report_plots",
        },
    ),
    "ReportPlot": (
        "NamedObject",
        {
            FluentVersion.v252: "solution.monitor.report_plots",
            FluentVersion.v251: "solution.monitor.report_plots",
            FluentVersion.v242: "solution.monitor.report_plots",
            FluentVersion.v241: "solution.monitor.report_plots",
            FluentVersion.v232: "solution.monitor.report_plots",
            FluentVersion.v231: "solution.monitor.report_plots",
        },
    ),
    "ConvergenceConditions": (
        "Singleton",
        {
            FluentVersion.v252: "solution.monitor.convergence_conditions",
            FluentVersion.v251: "solution.monitor.convergence_conditions",
            FluentVersion.v242: "solution.monitor.convergence_conditions",
            FluentVersion.v241: "solution.monitor.convergence_conditions",
            FluentVersion.v232: "solution.monitor.convergence_conditions",
            FluentVersion.v231: "solution.monitor.convergence_conditions",
        },
    ),
    "CellRegisters": (
        "Singleton",
        {
            FluentVersion.v252: "solution.cell_registers",
            FluentVersion.v251: "solution.cell_registers",
            FluentVersion.v242: "solution.cell_registers",
            FluentVersion.v241: "solution.cell_registers",
            FluentVersion.v232: "solution.cell_registers",
            FluentVersion.v231: "solution.cell_registers",
        },
    ),
    "CellRegister": (
        "NamedObject",
        {
            FluentVersion.v252: "solution.cell_registers",
            FluentVersion.v251: "solution.cell_registers",
            FluentVersion.v242: "solution.cell_registers",
            FluentVersion.v241: "solution.cell_registers",
            FluentVersion.v232: "solution.cell_registers",
            FluentVersion.v231: "solution.cell_registers",
        },
    ),
    "Initialization": ("Singleton", "solution.initialization"),
    "CalculationActivity": (
        "Singleton",
        {
            FluentVersion.v252: "solution.calculation_activity",
            FluentVersion.v251: "solution.calculation_activity",
            FluentVersion.v242: "solution.calculation_activity",
            FluentVersion.v241: "solution.calculation_activity",
            FluentVersion.v232: "solution.calculation_activity",
            FluentVersion.v231: "solution.calculation_activity",
        },
    ),
    "ExecuteCommands": (
        "Singleton",
        {
            FluentVersion.v252: "solution.calculation_activity.execute_commands",
            FluentVersion.v251: "solution.calculation_activity.execute_commands",
            FluentVersion.v242: "solution.calculation_activity.execute_commands",
            FluentVersion.v241: "solution.calculation_activity.execute_commands",
            FluentVersion.v232: "solution.calculation_activity.execute_commands",
            FluentVersion.v231: "solution.calculation_activity.execute_commands",
        },
    ),
    "CaseModification": (
        "Singleton",
        {
            FluentVersion.v252: "solution.calculation_activity.case_modification",
            FluentVersion.v251: "solution.calculation_activity.case_modification",
            FluentVersion.v242: "solution.calculation_activity.case_modification",
            FluentVersion.v241: "solution.calculation_activity.case_modification",
        },
    ),
    "RunCalculation": ("Singleton", "solution.run_calculation"),
    "Results": ("Singleton", "results"),
    "Surfaces": ("Singleton", "results.surfaces"),
    "PointSurfaces": (
        "Singleton",
        {
            FluentVersion.v252: "results.surfaces.point_surface",
            FluentVersion.v251: "results.surfaces.point_surface",
            FluentVersion.v242: "results.surfaces.point_surface",
            FluentVersion.v241: "results.surfaces.point_surface",
            FluentVersion.v232: "results.surfaces.point_surface",
        },
    ),
    "PointSurface": (
        "NamedObject",
        {
            FluentVersion.v252: "results.surfaces.point_surface",
            FluentVersion.v251: "results.surfaces.point_surface",
            FluentVersion.v242: "results.surfaces.point_surface",
            FluentVersion.v241: "results.surfaces.point_surface",
            FluentVersion.v232: "results.surfaces.point_surface",
        },
    ),
    "LineSurfaces": (
        "Singleton",
        {
            FluentVersion.v252: "results.surfaces.line_surface",
            FluentVersion.v251: "results.surfaces.line_surface",
            FluentVersion.v242: "results.surfaces.line_surface",
            FluentVersion.v241: "results.surfaces.line_surface",
            FluentVersion.v232: "results.surfaces.line_surface",
        },
    ),
    "LineSurface": (
        "NamedObject",
        {
            FluentVersion.v252: "results.surfaces.line_surface",
            FluentVersion.v251: "results.surfaces.line_surface",
            FluentVersion.v242: "results.surfaces.line_surface",
            FluentVersion.v241: "results.surfaces.line_surface",
            FluentVersion.v232: "results.surfaces.line_surface",
        },
    ),
    "RakeSurfaces": (
        "Singleton",
        {
            FluentVersion.v252: "results.surfaces.rake_surface",
            FluentVersion.v251: "results.surfaces.rake_surface",
            FluentVersion.v242: "results.surfaces.rake_surface",
            FluentVersion.v241: "results.surfaces.rake_surface",
            FluentVersion.v232: "results.surfaces.rake_surface",
        },
    ),
    "RakeSurface": (
        "NamedObject",
        {
            FluentVersion.v252: "results.surfaces.rake_surface",
            FluentVersion.v251: "results.surfaces.rake_surface",
            FluentVersion.v242: "results.surfaces.rake_surface",
            FluentVersion.v241: "results.surfaces.rake_surface",
            FluentVersion.v232: "results.surfaces.rake_surface",
        },
    ),
    "PlaneSurfaces": ("Singleton", "results.surfaces.plane_surface"),
    "PlaneSurface": ("NamedObject", "results.surfaces.plane_surface"),
    "IsoSurfaces": (
        "Singleton",
        {
            FluentVersion.v252: "results.surfaces.iso_surface",
            FluentVersion.v251: "results.surfaces.iso_surface",
            FluentVersion.v242: "results.surfaces.iso_surface",
            FluentVersion.v241: "results.surfaces.iso_surface",
            FluentVersion.v232: "results.surfaces.iso_surface",
        },
    ),
    "IsoSurface": (
        "NamedObject",
        {
            FluentVersion.v252: "results.surfaces.iso_surface",
            FluentVersion.v251: "results.surfaces.iso_surface",
            FluentVersion.v242: "results.surfaces.iso_surface",
            FluentVersion.v241: "results.surfaces.iso_surface",
            FluentVersion.v232: "results.surfaces.iso_surface",
        },
    ),
    "IsoClips": (
        "Singleton",
        {
            FluentVersion.v252: "results.surfaces.iso_clip",
            FluentVersion.v251: "results.surfaces.iso_clip",
            FluentVersion.v242: "results.surfaces.iso_clip",
            FluentVersion.v241: "results.surfaces.iso_clip",
        },
    ),
    "IsoClip": (
        "NamedObject",
        {
            FluentVersion.v252: "results.surfaces.iso_clip",
            FluentVersion.v251: "results.surfaces.iso_clip",
            FluentVersion.v242: "results.surfaces.iso_clip",
            FluentVersion.v241: "results.surfaces.iso_clip",
        },
    ),
    "ZoneSurfaces": (
        "Singleton",
        {
            FluentVersion.v252: "results.surfaces.zone_surface",
            FluentVersion.v251: "results.surfaces.zone_surface",
            FluentVersion.v242: "results.surfaces.zone_surface",
            FluentVersion.v241: "results.surfaces.zone_surface",
        },
    ),
    "ZoneSurface": (
        "NamedObject",
        {
            FluentVersion.v252: "results.surfaces.zone_surface",
            FluentVersion.v251: "results.surfaces.zone_surface",
            FluentVersion.v242: "results.surfaces.zone_surface",
            FluentVersion.v241: "results.surfaces.zone_surface",
        },
    ),
    "PartitionSurfaces": (
        "Singleton",
        {
            FluentVersion.v252: "results.surfaces.partition_surface",
            FluentVersion.v251: "results.surfaces.partition_surface",
            FluentVersion.v242: "results.surfaces.partition_surface",
            FluentVersion.v241: "results.surfaces.partition_surface",
        },
    ),
    "PartitionSurface": (
        "NamedObject",
        {
            FluentVersion.v252: "results.surfaces.partition_surface",
            FluentVersion.v251: "results.surfaces.partition_surface",
            FluentVersion.v242: "results.surfaces.partition_surface",
            FluentVersion.v241: "results.surfaces.partition_surface",
        },
    ),
    "TransformSurfaces": (
        "Singleton",
        {
            FluentVersion.v252: "results.surfaces.transform_surface",
            FluentVersion.v251: "results.surfaces.transform_surface",
            FluentVersion.v242: "results.surfaces.transform_surface",
            FluentVersion.v241: "results.surfaces.transform_surface",
        },
    ),
    "TransformSurface": (
        "NamedObject",
        {
            FluentVersion.v252: "results.surfaces.transform_surface",
            FluentVersion.v251: "results.surfaces.transform_surface",
            FluentVersion.v242: "results.surfaces.transform_surface",
            FluentVersion.v241: "results.surfaces.transform_surface",
        },
    ),
    "ImprintSurfaces": (
        "Singleton",
        {
            FluentVersion.v252: "results.surfaces.imprint_surface",
            FluentVersion.v251: "results.surfaces.imprint_surface",
            FluentVersion.v242: "results.surfaces.imprint_surface",
            FluentVersion.v241: "results.surfaces.imprint_surface",
        },
    ),
    "ImprintSurface": (
        "NamedObject",
        {
            FluentVersion.v252: "results.surfaces.imprint_surface",
            FluentVersion.v251: "results.surfaces.imprint_surface",
            FluentVersion.v242: "results.surfaces.imprint_surface",
            FluentVersion.v241: "results.surfaces.imprint_surface",
        },
    ),
    "PlaneSlices": (
        "Singleton",
        {
            FluentVersion.v252: "results.surfaces.plane_slice",
            FluentVersion.v251: "results.surfaces.plane_slice",
            FluentVersion.v242: "results.surfaces.plane_slice",
            FluentVersion.v241: "results.surfaces.plane_slice",
        },
    ),
    "PlaneSlice": (
        "NamedObject",
        {
            FluentVersion.v252: "results.surfaces.plane_slice",
            FluentVersion.v251: "results.surfaces.plane_slice",
            FluentVersion.v242: "results.surfaces.plane_slice",
            FluentVersion.v241: "results.surfaces.plane_slice",
        },
    ),
    "SphereSlices": (
        "Singleton",
        {
            FluentVersion.v252: "results.surfaces.sphere_slice",
            FluentVersion.v251: "results.surfaces.sphere_slice",
            FluentVersion.v242: "results.surfaces.sphere_slice",
            FluentVersion.v241: "results.surfaces.sphere_slice",
        },
    ),
    "SphereSlice": (
        "NamedObject",
        {
            FluentVersion.v252: "results.surfaces.sphere_slice",
            FluentVersion.v251: "results.surfaces.sphere_slice",
            FluentVersion.v242: "results.surfaces.sphere_slice",
            FluentVersion.v241: "results.surfaces.sphere_slice",
        },
    ),
    "QuadricSurfaces": (
        "Singleton",
        {
            FluentVersion.v252: "results.surfaces.quadric_surface",
            FluentVersion.v251: "results.surfaces.quadric_surface",
            FluentVersion.v242: "results.surfaces.quadric_surface",
            FluentVersion.v241: "results.surfaces.quadric_surface",
        },
    ),
    "QuadricSurface": (
        "NamedObject",
        {
            FluentVersion.v252: "results.surfaces.quadric_surface",
            FluentVersion.v251: "results.surfaces.quadric_surface",
            FluentVersion.v242: "results.surfaces.quadric_surface",
            FluentVersion.v241: "results.surfaces.quadric_surface",
        },
    ),
    "SurfaceCells": (
        "Singleton",
        {
            FluentVersion.v252: "results.surfaces.surface_cells",
            FluentVersion.v251: "results.surfaces.surface_cells",
            FluentVersion.v242: "results.surfaces.surface_cells",
            FluentVersion.v241: "results.surfaces.surface_cells",
        },
    ),
    "SurfaceCell": (
        "NamedObject",
        {
            FluentVersion.v252: "results.surfaces.surface_cells",
            FluentVersion.v251: "results.surfaces.surface_cells",
            FluentVersion.v242: "results.surfaces.surface_cells",
            FluentVersion.v241: "results.surfaces.surface_cells",
        },
    ),
    "ExpressionVolumes": (
        "Singleton",
        {
            FluentVersion.v252: "results.surfaces.expression_volume",
            FluentVersion.v251: "results.surfaces.expression_volume",
        },
    ),
    "ExpressionVolume": (
        "NamedObject",
        {
            FluentVersion.v252: "results.surfaces.expression_volume",
            FluentVersion.v251: "results.surfaces.expression_volume",
        },
    ),
    "GroupSurfaces": (
        "Singleton",
        {
            FluentVersion.v252: "results.surfaces.group_surface",
            FluentVersion.v251: "results.surfaces.group_surface",
        },
    ),
    "GroupSurface": (
        "NamedObject",
        {
            FluentVersion.v252: "results.surfaces.group_surface",
            FluentVersion.v251: "results.surfaces.group_surface",
        },
    ),
    "Graphics": ("Singleton", "results.graphics"),
    "Meshes": ("Singleton", "results.graphics.mesh"),
    "Mesh": ("NamedObject", "results.graphics.mesh"),
    "Contours": ("Singleton", "results.graphics.contour"),
    "Contour": ("NamedObject", "results.graphics.contour"),
    "Vectors": ("Singleton", "results.graphics.vector"),
    "Vector": ("NamedObject", "results.graphics.vector"),
    "Pathlines": (
        "Singleton",
        {
            FluentVersion.v252: "results.graphics.pathline",
            FluentVersion.v251: "results.graphics.pathline",
            FluentVersion.v242: "results.graphics.pathline",
            FluentVersion.v241: "results.graphics.pathline",
            FluentVersion.v232: "results.graphics.pathline",
            FluentVersion.v231: "results.graphics.pathline",
        },
    ),
    "Pathline": (
        "NamedObject",
        {
            FluentVersion.v252: "results.graphics.pathline",
            FluentVersion.v251: "results.graphics.pathline",
            FluentVersion.v242: "results.graphics.pathline",
            FluentVersion.v241: "results.graphics.pathline",
            FluentVersion.v232: "results.graphics.pathline",
            FluentVersion.v231: "results.graphics.pathline",
        },
    ),
    "ParticleTracks": (
        "Singleton",
        {
            FluentVersion.v252: "results.graphics.particle_track",
            FluentVersion.v251: "results.graphics.particle_track",
            FluentVersion.v242: "results.graphics.particle_track",
            FluentVersion.v241: "results.graphics.particle_track",
            FluentVersion.v232: "results.graphics.particle_track",
            FluentVersion.v231: "results.graphics.particle_track",
        },
    ),
    "ParticleTrack": (
        "NamedObject",
        {
            FluentVersion.v252: "results.graphics.particle_track",
            FluentVersion.v251: "results.graphics.particle_track",
            FluentVersion.v242: "results.graphics.particle_track",
            FluentVersion.v241: "results.graphics.particle_track",
            FluentVersion.v232: "results.graphics.particle_track",
            FluentVersion.v231: "results.graphics.particle_track",
        },
    ),
    "LICs": ("Singleton", "results.graphics.lic"),
    "LIC": ("NamedObject", "results.graphics.lic"),
    "Plots": (
        "Singleton",
        {
            FluentVersion.v252: "results.plot",
            FluentVersion.v251: "results.plot",
            FluentVersion.v242: "results.plot",
            FluentVersion.v241: "results.plot",
            FluentVersion.v232: "results.plot",
            FluentVersion.v231: "results.plot",
        },
    ),
    "XYPlots": (
        "Singleton",
        {
            FluentVersion.v252: "results.plot.xy_plot",
            FluentVersion.v251: "results.plot.xy_plot",
            FluentVersion.v242: "results.plot.xy_plot",
            FluentVersion.v241: "results.plot.xy_plot",
            FluentVersion.v232: "results.plot.xy_plot",
            FluentVersion.v231: "results.plot.xy_plot",
        },
    ),
    "XYPlot": (
        "NamedObject",
        {
            FluentVersion.v252: "results.plot.xy_plot",
            FluentVersion.v251: "results.plot.xy_plot",
            FluentVersion.v242: "results.plot.xy_plot",
            FluentVersion.v241: "results.plot.xy_plot",
            FluentVersion.v232: "results.plot.xy_plot",
            FluentVersion.v231: "results.plot.xy_plot",
        },
    ),
    "Histogram": (
        "Singleton",
        {
            FluentVersion.v252: "results.plot.histogram",
            FluentVersion.v251: "results.plot.histogram",
            FluentVersion.v242: "results.plot.histogram",
            FluentVersion.v241: "results.plot.histogram",
        },
    ),
    "CumulativePlots": (
        "Singleton",
        {
            FluentVersion.v252: "results.plot.cumulative_plot",
            FluentVersion.v251: "results.plot.cumulative_plot",
            FluentVersion.v242: "results.plot.cumulative_plot",
            FluentVersion.v241: "results.plot.cumulative_plot",
        },
    ),
    "CumulativePlot": (
        "NamedObject",
        {
            FluentVersion.v252: "results.plot.cumulative_plot",
            FluentVersion.v251: "results.plot.cumulative_plot",
            FluentVersion.v242: "results.plot.cumulative_plot",
            FluentVersion.v241: "results.plot.cumulative_plot",
        },
    ),
    "ProfileData": (
        "Singleton",
        {
            FluentVersion.v252: "results.plot.profile_data",
            FluentVersion.v251: "results.plot.profile_data",
            FluentVersion.v242: "results.plot.profile_data",
        },
    ),
    "InterpolatedData": (
        "Singleton",
        {
            FluentVersion.v252: "results.plot.interpolated_data",
            FluentVersion.v251: "results.plot.interpolated_data",
            FluentVersion.v242: "results.plot.interpolated_data",
        },
    ),
    "Scenes": (
        "Singleton",
        {
            FluentVersion.v252: "results.scene",
            FluentVersion.v251: "results.scene",
            FluentVersion.v242: "results.scene",
            FluentVersion.v241: "results.scene",
            FluentVersion.v232: "results.scene",
            FluentVersion.v231: "results.scene",
        },
    ),
    "Scene": (
        "NamedObject",
        {
            FluentVersion.v252: "results.scene",
            FluentVersion.v251: "results.scene",
            FluentVersion.v242: "results.scene",
            FluentVersion.v241: "results.scene",
            FluentVersion.v232: "results.scene",
            FluentVersion.v231: "results.scene",
        },
    ),
    "SceneAnimation": (
        "Singleton",
        {
            FluentVersion.v252: "results.animations.scene_animation",
            FluentVersion.v251: "results.animations.scene_animation",
            FluentVersion.v242: "results.animations.scene_animation",
            FluentVersion.v241: "results.animations.scene_animation",
        },
    ),
    "Report": (
        "Singleton",
        {
            FluentVersion.v252: "results.report",
            FluentVersion.v251: "results.report",
            FluentVersion.v242: "results.report",
            FluentVersion.v241: "results.report",
            FluentVersion.v232: "results.report",
            FluentVersion.v231: "results.report",
        },
    ),
    "DiscretePhaseHistogram": (
        "Singleton",
        {
            FluentVersion.v252: "results.report.discrete_phase.histogram",
            FluentVersion.v251: "results.report.discrete_phase.histogram",
            FluentVersion.v242: "results.report.discrete_phase.histogram",
            FluentVersion.v241: "results.report.discrete_phase.histogram",
            FluentVersion.v232: "results.report.discrete_phase.histogram",
            FluentVersion.v231: "results.report.discrete_phase.histogram",
        },
    ),
    "Fluxes": (
        "Singleton",
        {
            FluentVersion.v252: "results.report.fluxes",
            FluentVersion.v251: "results.report.fluxes",
            FluentVersion.v242: "results.report.fluxes",
            FluentVersion.v241: "results.report.fluxes",
            FluentVersion.v232: "results.report.fluxes",
            FluentVersion.v231: "results.report.fluxes",
        },
    ),
    "SurfaceIntegrals": (
        "Singleton",
        {
            FluentVersion.v252: "results.report.surface_integrals",
            FluentVersion.v251: "results.report.surface_integrals",
            FluentVersion.v242: "results.report.surface_integrals",
            FluentVersion.v241: "results.report.surface_integrals",
            FluentVersion.v232: "results.report.surface_integrals",
            FluentVersion.v231: "results.report.surface_integrals",
        },
    ),
    "VolumeIntegrals": (
        "Singleton",
        {
            FluentVersion.v252: "results.report.volume_integrals",
            FluentVersion.v251: "results.report.volume_integrals",
            FluentVersion.v242: "results.report.volume_integrals",
            FluentVersion.v241: "results.report.volume_integrals",
            FluentVersion.v232: "results.report.volume_integrals",
            FluentVersion.v231: "results.report.volume_integrals",
        },
    ),
    "InputParameters": (
        "Singleton",
        {
            FluentVersion.v252: "parameters.input_parameters",
            FluentVersion.v251: "parameters.input_parameters",
            FluentVersion.v242: "parameters.input_parameters",
            FluentVersion.v241: "parameters.input_parameters",
        },
    ),
    "OutputParameters": (
        "Singleton",
        {
            FluentVersion.v252: "parameters.output_parameters",
            FluentVersion.v251: "parameters.output_parameters",
            FluentVersion.v242: "parameters.output_parameters",
            FluentVersion.v241: "parameters.output_parameters",
        },
    ),
    "CustomFieldFunctions": (
        "Singleton",
        {
            FluentVersion.v252: "results.custom_field_functions",
            FluentVersion.v251: "results.custom_field_functions",
        },
    ),
    "CustomFieldFunction": (
        "NamedObject",
        {
            FluentVersion.v252: "results.custom_field_functions",
            FluentVersion.v251: "results.custom_field_functions",
        },
    ),
    "CustomVectors": (
        "Singleton",
        {
            FluentVersion.v252: "results.custom_vectors",
            FluentVersion.v251: "results.custom_vectors",
            FluentVersion.v242: "results.custom_vectors",
            FluentVersion.v241: "results.custom_vectors",
        },
    ),
    "CustomVector": (
        "NamedObject",
        {
            FluentVersion.v252: "results.custom_vectors",
            FluentVersion.v251: "results.custom_vectors",
            FluentVersion.v242: "results.custom_vectors",
            FluentVersion.v241: "results.custom_vectors",
        },
    ),
    "SimulationReports": (
        "Singleton",
        {
            FluentVersion.v252: "results.report.simulation_reports",
            FluentVersion.v251: "results.report.simulation_reports",
            FluentVersion.v242: "results.report.simulation_reports",
            FluentVersion.v241: "results.report.simulation_reports",
            FluentVersion.v232: "results.report.simulation_reports",
            FluentVersion.v231: "results.report.simulation_reports",
        },
    ),
    "ParametricStudies": ("Singleton", "parametric_studies"),
    "ParametricStudy": ("NamedObject", "parametric_studies"),
    "DesignPoints": ("Singleton", "parametric_studies.design_points"),
    "DesignPoint": ("NamedObject", "parametric_studies.design_points"),
}



# Copyright (C) 2021 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import pytest

from ansys.fluent.core.utils.fluent_version import AnsysVersionNotFound, FluentVersion


def test_examples():
    assert FluentVersion("23.2.0") == FluentVersion.v232
    assert FluentVersion.v232.number == 232
    assert FluentVersion.v232.awp_var == "AWP_ROOT232"
    assert FluentVersion.v232.name == "v232"
    assert FluentVersion.v232.value == "23.2.0"


def test_version_found():
    assert FluentVersion("23.2") == FluentVersion.v232
    assert FluentVersion(23.2) == FluentVersion.v232
    assert FluentVersion(232) == FluentVersion.v232


def test_version_not_found():
    with pytest.raises(AnsysVersionNotFound):
        FluentVersion("25.3.0")

    with pytest.raises(AnsysVersionNotFound):
        FluentVersion(22)


def test_get_latest_installed(helpers):
    helpers.mock_awp_vars()
    assert FluentVersion.get_latest_installed() == FluentVersion.current_release()


def test_gt():
    assert FluentVersion.v232 > FluentVersion.v231
    assert FluentVersion.v232 > FluentVersion.v222


def test_ge():
    assert FluentVersion.v232 >= FluentVersion.v232
    assert FluentVersion.v232 >= FluentVersion.v231
    assert FluentVersion.v232 >= FluentVersion.v222


def test_lt():
    assert FluentVersion.v232 < FluentVersion.v242
    assert FluentVersion.v232 < FluentVersion.v241


def test_le():
    assert FluentVersion.v232 <= FluentVersion.v232
    assert FluentVersion.v232 <= FluentVersion.v242
    assert FluentVersion.v232 <= FluentVersion.v241


def test_ne():
    assert FluentVersion.v232 != FluentVersion.v242


def test_eq():
    assert FluentVersion.v232 == FluentVersion.v232
    assert FluentVersion.v241 == FluentVersion.v241


def test_str_output():
    assert str(FluentVersion.v232) == "Fluent version 2023 R2"
    assert str(FluentVersion.v251) == "Fluent version 2025 R1"



# Copyright (C) 2021 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"Test DataModelAPIService through journal."


def create_datamodel_root_in_server(solver) -> None:
    import uuid

    app_name = "test"
    rules_str = (
        "RULES:\n"
        "  STRING: X\n"
        "    default = ijk\n"
        "  END\n"
        "  SINGLETON: ROOT\n"
        "    members = A, B, D, G\n"
        "    commands= C\n"
        "    SINGLETON: A\n"
        "      members = X\n"
        "      x = $./X\n"
        "    END\n"
        "    OBJECT: B\n"
        "      members = X\n"
        "    END\n"
        "    SINGLETON: D\n"
        "      members = E, F, X\n"
        "      SINGLETON: E\n"
        "        members = X\n"
        "      END\n"
        "      SINGLETON: F\n"
        "        members = X\n"
        "      END\n"
        "    END\n"
        "    SINGLETON: G\n"
        "      members = H\n"
        "      DICT: H\n"
        "      END\n"
        "    END\n"
        "    COMMAND: C\n"
        "      arguments = X\n"
        "      x = $/A/X\n"
        "    END\n"
        "  END\n"
        "END\n"
    )
    rules_file_name = f"{uuid.uuid4()}.fdl"
    solver.scheme_eval.scheme_eval(
        f'(with-output-to-file "{rules_file_name}" (lambda () (format "~a" "{rules_str}")))',
    )
    solver.scheme_eval.scheme_eval(
        f'(state/register-new-state-engine "{app_name}" "{rules_file_name}")'
    )
    solver.scheme_eval.scheme_eval(f'(remove-file "{rules_file_name}")')


def test_datamodel_api_on_child_created(solver):
    import time

    create_datamodel_root_in_server(solver)
    app_name = "test"
    service = solver._se_service
    called = 0
    created = []

    def cb(path):
        nonlocal called
        nonlocal created
        called += 1
        created.append(path)

    subscription = service.add_on_child_created(app_name, "/", "B", cb)
    assert called == 0
    assert created == []
    service.set_state(app_name, "/", {"B:b": {"_name_": "b"}})
    time.sleep(6)
    assert called == 1
    assert created == ["/B:b"]
    subscription.unsubscribe()


def test_datamodel_api_on_changed(solver):
    import time

    create_datamodel_root_in_server(solver)
    app_name = "test"
    service = solver._se_service
    called = 0
    changed = None
    called_obj = 0
    changed_obj = None

    def cb(path):
        nonlocal called
        nonlocal changed
        changed = path
        called += 1

    def cb_obj(path):
        nonlocal called_obj
        nonlocal changed_obj
        changed_obj = path
        called_obj += 1

    subscription = service.add_on_changed(app_name, "/A/X", cb)
    subscription_obj = service.add_on_changed(app_name, "/A", cb_obj)
    assert called == 0
    assert changed is None
    assert called_obj == 0
    assert changed_obj is None
    service.set_state(app_name, "/A/X", "lmn")
    time.sleep(5)
    assert called == 1
    assert changed == "lmn"
    assert called_obj == 1
    assert changed_obj == {"X": "lmn"}
    service.set_state(app_name, "/A/X", "abc")
    time.sleep(5)
    assert called == 2
    assert changed == "abc"
    assert called_obj == 2
    assert changed_obj == {"X": "abc"}
    subscription.unsubscribe()
    subscription_obj.unsubscribe()
    service.set_state(app_name, "/A/X", "xyz")
    time.sleep(5)
    assert called == 2
    assert changed == "abc"
    assert called_obj == 2
    assert changed_obj == {"X": "abc"}


def test_datamodel_api_on_attribute_changed(solver):
    import time

    create_datamodel_root_in_server(solver)
    app_name = "test"
    service = solver._se_service
    called = 0
    value = None

    def cb(attribute):
        nonlocal called
        nonlocal value
        value = attribute
        called += 1

    subscription = service.add_on_attribute_changed(app_name, "/A", "x", cb)
    assert called == 0
    assert value is None
    service.set_state(app_name, "/A/X", "cde")
    time.sleep(5)
    assert called == 1
    assert value == "cde"
    service.set_state(app_name, "/A/X", "xyz")
    time.sleep(5)
    assert called == 2
    assert value == "xyz"
    subscription.unsubscribe()
    service.set_state(app_name, "/A/X", "abc")
    time.sleep(5)
    assert called == 2
    assert value == "xyz"


def test_datamodel_api_on_command_attribute_changed(solver):
    import time

    create_datamodel_root_in_server(solver)
    app_name = "test"
    service = solver._se_service
    called = 0
    value = None

    def cb(val):
        nonlocal called
        nonlocal value
        value = val
        called += 1

    subscription = service.add_on_command_attribute_changed(app_name, "/", "C", "x", cb)
    assert called == 0
    assert value is None
    service.set_state(app_name, "/A/X", "cde")
    time.sleep(5)
    assert called == 1
    assert value == "cde"
    service.set_state(app_name, "/A/X", "xyz")
    time.sleep(5)
    assert called == 2
    assert value == "xyz"
    subscription.unsubscribe()
    service.set_state(app_name, "/A/X", "abc")
    time.sleep(5)
    assert called == 2
    assert value == "xyz"


def test_datamodel_api_on_command_executed(solver):
    import time

    create_datamodel_root_in_server(solver)
    app_name = "test"
    service = solver._se_service
    executed = 0
    command = None
    arguments = None

    def cb(cmd, args):
        nonlocal executed
        nonlocal command
        nonlocal arguments
        command = cmd
        arguments = args
        executed += 1

    subscription = service.add_on_command_executed(app_name, "/", cb)
    assert executed == 0
    assert command is None
    assert arguments is None
    service.execute_command(app_name, "/", "C", dict(X="abc"))
    time.sleep(5)
    assert executed == 1
    assert command == "C"
    assert arguments == {"X": "abc"}
    subscription.unsubscribe()
    service.execute_command(app_name, "/", "C", dict(X="uvw"))
    time.sleep(5)
    assert executed == 1
    assert command == "C"
    assert arguments == {"X": "abc"}


def test_datamodel_api_get_state(solver):
    create_datamodel_root_in_server(solver)
    app_name = "test"
    service = solver._se_service
    assert service.get_state(app_name, "/A/X") == "ijk"


def test_datamodel_api_set_state(solver):
    create_datamodel_root_in_server(solver)
    app_name = "test"
    service = solver._se_service
    service.set_state(app_name, "/A/X", "new_val")
    assert service.get_state(app_name, "/A/X") == "new_val"


def test_datamodel_api_update_dict(solver):
    create_datamodel_root_in_server(solver)
    app_name = "test"
    service = solver._se_service
    service.update_dict(app_name, "/G/H", {"X": "abc"})
    assert service.get_state(app_name, "/G/H") == {"X": "abc"}


def test_datamodel_api_static_info(solver):
    create_datamodel_root_in_server(solver)
    app_name = "test"
    service = solver._se_service
    assert service.get_static_info(app_name)


def test_datamodel_api_on_affected(solver):
    import time

    create_datamodel_root_in_server(solver)
    app_name = "test"
    service = solver._se_service
    called = 0

    def cb():
        nonlocal called
        called += 1

    subscription = service.add_on_affected(app_name, "/D", cb)
    assert called == 0
    service.set_state(app_name, "/D/X", "lmn")
    time.sleep(5)
    assert called == 1
    service.set_state(app_name, "/D/E/X", "lmn")
    time.sleep(5)
    assert called == 2
    service.set_state(app_name, "/A/X", "lmn")
    time.sleep(5)
    assert called == 2
    subscription.unsubscribe()
    service.set_state(app_name, "/D/E/X", "pqr")
    time.sleep(5)
    assert called == 2


def test_datamodel_api_on_affected_at_type_path(solver):
    import time

    create_datamodel_root_in_server(solver)
    app_name = "test"
    service = solver._se_service
    called = 0

    def cb():
        nonlocal called
        called += 1

    subscription = service.add_on_affected_at_type_path(app_name, "/D", "E", cb)
    assert called == 0
    service.set_state(app_name, "/D/X", "lmn")
    time.sleep(5)
    assert called == 0
    service.set_state(app_name, "/D/E/X", "lmn")
    time.sleep(5)
    assert called == 1
    service.set_state(app_name, "/D/F/X", "lmn")
    time.sleep(5)
    assert called == 1
    subscription.unsubscribe()
    service.set_state(app_name, "/D/E/X", "pqr")
    time.sleep(5)
    assert called == 1


def test_datamodel_api_on_deleted(solver):
    import time

    create_datamodel_root_in_server(solver)
    app_name = "test"
    service = solver._se_service
    called = False
    called_obj = False

    def cb():
        nonlocal called
        called = True

    def cb_obj():
        nonlocal called_obj
        called_obj = True

    service.set_state(app_name, "/", {"B:b": {"_name_": "b"}})
    subscription = service.add_on_deleted(app_name, "/B:b/X", cb)
    subscription_obj = service.add_on_deleted(app_name, "/B:b", cb_obj)
    assert not called
    assert not called_obj
    service.delete_object(app_name, "/B:b")
    time.sleep(5)
    assert not called
    assert called_obj
    subscription.unsubscribe()
    subscription_obj.unsubscribe()


def test_datamodel_api_rename_and_get_object_names(solver):
    create_datamodel_root_in_server(solver)
    app_name = "test"
    service = solver._se_service

    def cb(path):
        pass

    subscription = service.add_on_child_created(app_name, "/", "B", cb)
    service.set_state(app_name, "/", {"B:b": {"_name_": "b"}})
    service.rename(app_name, "/B:b", "b_new")
    assert service.get_object_names(app_name, "/B:b_new") == ["b_new"]
    subscription.unsubscribe()


create_datamodel_root_in_server(solver)  # noqa: F821
test_datamodel_api_on_child_created(solver)  # noqa: F821
test_datamodel_api_on_changed(solver)  # noqa: F821
test_datamodel_api_on_attribute_changed(solver)  # noqa: F821
test_datamodel_api_on_command_attribute_changed(solver)  # noqa: F821
test_datamodel_api_on_command_executed(solver)  # noqa: F821
test_datamodel_api_get_state(solver)  # noqa: F821
test_datamodel_api_set_state(solver)  # noqa: F821
test_datamodel_api_update_dict(solver)  # noqa: F821
test_datamodel_api_static_info(solver)  # noqa: F821
test_datamodel_api_on_affected(solver)  # noqa: F821
test_datamodel_api_on_affected_at_type_path(solver)  # noqa: F821
test_datamodel_api_on_deleted(solver)  # noqa: F821
test_datamodel_api_rename_and_get_object_names(solver)  # noqa: F821

print("\n Testing finished. All tests passed.\n")



# Copyright (C) 2021 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Module to generate Fluent datamodel API classes."""

import argparse
from io import FileIO, StringIO
import os
from pathlib import Path
import shutil
from typing import Any, Dict

import ansys.fluent.core as pyfluent
from ansys.fluent.core import FluentMode, launch_fluent
from ansys.fluent.core.codegen import StaticInfoType
from ansys.fluent.core.codegen.data.meshing_utilities_examples import (
    meshing_utility_examples,
)
from ansys.fluent.core.utils.fluent_version import (
    FluentVersion,
    get_version_for_file_name,
)

_ROOT_DIR = Path(__file__) / ".." / ".." / ".." / ".." / ".." / ".."

_PY_TYPE_BY_DM_TYPE = {
    **dict.fromkeys(["Logical", "Bool"], "bool"),
    **dict.fromkeys(["Logical List", "ListBool"], "list[bool]"),
    "String": "str",
    **dict.fromkeys(["String List", "ListString"], "list[str]"),
    **dict.fromkeys(["Integer", "Int"], "int"),
    **dict.fromkeys(["Integer List", "ListInt"], "list[int]"),
    "Real": "float",
    **dict.fromkeys(
        [
            "Real List",
            "ListReal",
            "Real Triplet",
            "RealTriplet",
            "Real Triplet List",
            "ListRealTriplet",
        ],
        "list[float]",
    ),
    **dict.fromkeys(["Dict", "ModelObject"], "dict[str, Any]"),
    "None": "None",
}

# TODO: Move doc specific variables to docgen

_MESHING_DM_DOC_DIR = os.path.normpath(
    os.path.join(
        _ROOT_DIR,
        "doc",
        "source",
        "api",
        "meshing",
        "datamodel",
    )
)
_SOLVER_DM_DOC_DIR = os.path.normpath(
    os.path.join(
        _ROOT_DIR,
        "doc",
        "source",
        "api",
        "solver",
        "datamodel",
    )
)


def _write_command_query_stub(name: str, info: Any, f: FileIO):
    signature = StringIO()
    indent = "        "
    signature.write(f"(\n{indent}self,\n")
    if info.get("args"):
        for arg in info.get("args"):
            signature.write(
                f'{indent}{arg["name"]}: {_PY_TYPE_BY_DM_TYPE[arg["type"]]} | None = None,\n'
            )
    signature.write(f'{indent}) -> {_PY_TYPE_BY_DM_TYPE[info["returntype"]]}: ...')
    f.write(f"\n    def {name}{signature.getvalue()}\n")


def _build_singleton_docstring(name: str):
    return f"Singleton {name}."


def _build_parameter_docstring(name: str, t: str):
    return f"Parameter {name} of value type {_PY_TYPE_BY_DM_TYPE[t]}."


def _build_command_query_docstring(name: str, info: Any, indent: str, is_command: bool):
    if info.get("docstring"):
        doc = ""
        for line in info["docstring"].split("."):
            if line and len(info["docstring"].split(".")) > 2:
                doc += f"{indent}- {line.lstrip(' ')}.\n"
            elif line:
                doc += f"{indent}{line.lstrip(' ')}.\n"
        if info.get("args"):
            doc += "\n"
    else:
        doc = (
            f"{indent}Command {name}.\n\n"
            if is_command
            else f"{indent}Query {name}.\n\n"
        )
    if info.get("args"):
        doc += f"{indent}Parameters\n"
        doc += f"{indent}{'-' * len('Parameters')}\n"
        for arg in info.get("args"):
            doc += f'{indent}{arg["name"]} : {_PY_TYPE_BY_DM_TYPE[arg["type"]]}\n'
            if arg.get("docstring"):
                doc += f'{indent}    {arg["docstring"]}\n'
    doc += f"\n{indent}Returns\n"
    doc += f"{indent}{'-' * len('Returns')}\n"
    doc += f'{indent}{_PY_TYPE_BY_DM_TYPE[info["returntype"]]}\n'
    if meshing_utility_examples.get(name):
        doc += f"\n{indent}Examples\n"
        doc += f"{indent}{'-' * len('Examples')}\n"
        for example in meshing_utility_examples[name]:
            doc += f"{indent}>>> {example}\n"
    return doc


meshing_rule_file_names = {
    "workflow": "workflow",
    "meshing": "meshing",
    "PartManagement": "part_management",
    "PMFileManagement": "pm_file_management",
    "preferences": "preferences",
    "MeshingUtilities": "meshing_utilities",
    "flicing": "flicing",
    "solverworkflow": "solver_workflow",
}


class DataModelStaticInfo:
    """Stores datamodel static information."""

    _noindices = []

    def __init__(
        self,
        static_info_type: StaticInfoType,
        rules: str,
        modes: tuple,
        version: str,
        rules_save_name: str = "",
    ):
        self.static_info_type = static_info_type
        self.rules = rules
        self.modes = modes
        self.static_info = None
        if rules_save_name == "":
            rules_save_name = rules
        datamodel_dir = (pyfluent.CODEGEN_OUTDIR / f"datamodel_{version}").resolve()
        datamodel_dir.mkdir(exist_ok=True)
        self.file_name = (
            datamodel_dir / f"{meshing_rule_file_names[rules_save_name]}.py"
        ).resolve()
        if rules == "MeshingUtilities":
            self.stub_file = (datamodel_dir / "meshing_utilities.pyi").resolve()
        if len(modes) > 1:
            for mode in modes[1:]:
                DataModelStaticInfo._noindices.append(f"{mode}.datamodel.{rules}")


class DataModelGenerator:
    """Provides the datamodel API class generator."""

    def __init__(self, version, static_infos: dict, verbose: bool = False):
        self.version = version
        self._server_static_infos = static_infos
        self._static_info: Dict[str, DataModelStaticInfo] = {}
        self._verbose = verbose
        if StaticInfoType.DATAMODEL_WORKFLOW in static_infos:
            self._static_info["workflow"] = DataModelStaticInfo(
                StaticInfoType.DATAMODEL_WORKFLOW,
                "workflow",
                (
                    "meshing",
                    "solver",
                ),
                self.version,
            )
        if StaticInfoType.DATAMODEL_MESHING in static_infos:
            self._static_info["meshing"] = DataModelStaticInfo(
                StaticInfoType.DATAMODEL_MESHING, "meshing", ("meshing",), self.version
            )
        if StaticInfoType.DATAMODEL_PART_MANAGEMENT in static_infos:
            self._static_info["PartManagement"] = DataModelStaticInfo(
                StaticInfoType.DATAMODEL_PART_MANAGEMENT,
                "PartManagement",
                ("meshing",),
                self.version,
            )
        if StaticInfoType.DATAMODEL_PM_FILE_MANAGEMENT in static_infos:
            self._static_info["PMFileManagement"] = DataModelStaticInfo(
                StaticInfoType.DATAMODEL_PM_FILE_MANAGEMENT,
                "PMFileManagement",
                ("meshing",),
                self.version,
            )
        if StaticInfoType.DATAMODEL_FLICING in static_infos:
            self._static_info["flicing"] = DataModelStaticInfo(
                StaticInfoType.DATAMODEL_FLICING,
                "flserver",
                ("flicing",),
                self.version,
                "flicing",
            )
        if StaticInfoType.DATAMODEL_PREFERENCES in static_infos:
            self._static_info["preferences"] = DataModelStaticInfo(
                StaticInfoType.DATAMODEL_PREFERENCES,
                "preferences",
                ("meshing", "solver", "flicing"),
                self.version,
            )
        if StaticInfoType.DATAMODEL_SOLVER_WORKFLOW in static_infos:
            self._static_info["solverworkflow"] = DataModelStaticInfo(
                StaticInfoType.DATAMODEL_SOLVER_WORKFLOW,
                "solverworkflow",
                ("solver",),
                self.version,
            )
        if StaticInfoType.DATAMODEL_MESHING_UTILITIES in static_infos:
            self._static_info["MeshingUtilities"] = DataModelStaticInfo(
                StaticInfoType.DATAMODEL_MESHING_UTILITIES,
                "MeshingUtilities",
                ("meshing",),
                self.version,
            )
        self._delete_generated_files()
        self._populate_static_info()

    def _get_static_info(self, static_info_type: StaticInfoType):
        return self._server_static_infos[static_info_type]

    def _populate_static_info(self):
        run_meshing_mode = any(
            "meshing" in info.modes for _, info in self._static_info.items()
        )
        run_solver_mode = any(
            "solver" in info.modes for _, info in self._static_info.items()
        )
        run_icing_mode = FluentVersion(self.version) >= FluentVersion.v231 and any(
            "flicing" in info.modes for _, info in self._static_info.items()
        )

        if run_meshing_mode:
            for _, info in self._static_info.items():
                if "meshing" in info.modes:
                    info.static_info = self._get_static_info(info.static_info_type)

        if run_solver_mode:
            for _, info in self._static_info.items():
                if "solver" in info.modes:
                    info.static_info = self._get_static_info(info.static_info_type)

        if run_icing_mode:
            info = self._static_info.get("flicing")
            if info:
                info.static_info = self._get_static_info(
                    StaticInfoType.DATAMODEL_FLICING
                )
                try:
                    if (
                        len(
                            info.static_info["singletons"]["Case"]["singletons"]["App"][
                                "singletons"
                            ]
                        )
                        == 0
                    ):
                        print(
                            "Information: Icing settings not generated ( R23.1+ is required )\n"
                        )
                except Exception:
                    print(
                        "Information: Problem accessing flserver datamodel for icing settings\n"
                    )

    def _write_static_info(self, name: str, info: Any, f: FileIO, level: int = 0):
        api_tree = {}
        # preferences contains a deprecated object Meshing Workflow (with a space)
        # which migrates to MeshingWorkflow automatically. Simplest thing to do is
        # filter out invalid names.
        if not name.isidentifier():
            return api_tree
        indent = " " * level * 4
        f.write(f"{indent}class {name}(PyMenu):\n")
        f.write(f'{indent}    """\n')
        f.write(f"{indent}    {_build_singleton_docstring(name)}\n")
        f.write(f'{indent}    """\n')
        f.write(f"{indent}    def __init__(self, service, rules, path):\n")
        named_objects = sorted(info.get("namedobjects", []))
        singletons = sorted(info.get("singletons", []))
        parameters = sorted(info.get("parameters", []))
        commands = sorted(info.get("commands", []))
        queries = sorted(info.get("queries", []))
        for k in named_objects:
            f.write(
                f"{indent}        self.{k} = "
                f'self.__class__.{k}(service, rules, path + [("{k}", "")])\n'
            )
        for k in singletons:
            # This is where filtering these names out really matters (see commsent above)
            if k.isidentifier():
                f.write(
                    f"{indent}        self.{k} = "
                    f'self.__class__.{k}(service, rules, path + [("{k}", "")])\n'
                )
        for k in parameters:
            f.write(
                f"{indent}        self.{k} = "
                f'self.__class__.{k}(service, rules, path + [("{k}", "")])\n'
            )
        for k in commands:
            f.write(
                f"{indent}        self.{k} = "
                f'self.__class__.{k}(service, rules, "{k}", path)\n'
            )
        for k in queries:
            f.write(
                f"{indent}        self.{k} = "
                f'self.__class__.{k}(service, rules, "{k}", path)\n'
            )
        f.write(f"{indent}        super().__init__(service, rules, path)\n\n")
        for k in named_objects:
            f.write(f"{indent}    class {k}(PyNamedObjectContainer):\n")
            f.write(f'{indent}        """\n')
            f.write(f"{indent}        .\n")
            f.write(f'{indent}        """\n')
            api_tree[f"{k}:<name>"] = self._write_static_info(
                f"_{k}", info["namedobjects"][k], f, level + 2
            )
            # Specify the concrete named object type for __getitem__
            f.write(f"{indent}        def __getitem__(self, key: str) -> " f"_{k}:\n")
            f.write(f"{indent}            return super().__getitem__(key)\n\n")
        for k in singletons:
            if k.isidentifier():
                # print("included", k)
                api_tree[k] = self._write_static_info(
                    k, info["singletons"][k], f, level + 1
                )
            else:
                # print("\t\texcluded", k)
                pass
        for k in parameters:
            k_type = info["parameters"][k]["type"]
            if k_type in {"String", "String List", "ListString"}:
                f.write(f"{indent}    class {k}(PyTextual):\n")
            elif k_type in {"Integer", "Int", "Real"}:
                f.write(f"{indent}    class {k}(PyNumerical):\n")
            elif k_type in {"Dict", "ModelObject"}:
                f.write(f"{indent}    class {k}(PyDictionary):\n")
            else:
                f.write(f"{indent}    class {k}(PyParameter):\n")
            f.write(f'{indent}        """\n')
            f.write(
                f"{indent}        "
                f'{_build_parameter_docstring(k, info["parameters"][k]["type"])}\n'
            )
            f.write(f'{indent}        """\n')
            f.write(f"{indent}        pass\n\n")
            api_tree[k] = "Parameter"
        if "meshing_utilities" in f.name:
            stub_file = self._static_info["MeshingUtilities"].stub_file
            stub_file.unlink(missing_ok=True)
            with open(stub_file, "w", encoding="utf8") as file:
                file.write("#\n")
                file.write("# This is an auto-generated file.  DO NOT EDIT!\n")
                file.write("#\n")
                file.write("# pylint: disable=line-too-long\n\n")
                file.write(
                    "from ansys.fluent.core.services.datamodel_se import PyMenu\n"
                )
                file.write("from typing import Any\n")
                file.write("\n\n")
                file.write("class Root(PyMenu):\n")
                for k in commands:
                    _write_command_query_stub(
                        k,
                        info["commands"][k]["commandinfo"],
                        file,
                    )
                for k in queries:
                    _write_command_query_stub(
                        k,
                        info["queries"][k]["queryinfo"],
                        file,
                    )
        for k in commands:
            f.write(f"{indent}    class {k}(PyCommand):\n")
            f.write(f'{indent}        """\n')
            f.write(
                _build_command_query_docstring(
                    k, info["commands"][k]["commandinfo"], f"{indent}        ", True
                )
            )
            f.write(f'{indent}        """\n')
            f.write(f"{indent}        pass\n\n")
            api_tree[k] = "Command"
        for k in queries:
            f.write(f"{indent}    class {k}(PyQuery):\n")
            f.write(f'{indent}        """\n')
            f.write(
                _build_command_query_docstring(
                    k, info["queries"][k]["queryinfo"], f"{indent}        ", False
                )
            )
            f.write(f'{indent}        """\n')
            f.write(f"{indent}        pass\n\n")
            api_tree[k] = "Query"
        return api_tree

    def write_static_info(self) -> None:
        """Write API classes to files."""
        api_tree = {"<meshing_session>": {}, "<solver_session>": {}}
        for name, info in self._static_info.items():
            if self._verbose:
                print(f"{str(info.file_name)}")
            if info.static_info is None:
                continue
            with open(info.file_name, "w", encoding="utf8") as f:
                f.write("#\n")
                f.write("# This is an auto-generated file.  DO NOT EDIT!\n")
                f.write("#\n")
                f.write("# pylint: disable=line-too-long\n\n")
                f.write("from ansys.fluent.core.services.datamodel_se import (\n")
                f.write("    PyMenu,\n")
                f.write("    PyParameter,\n")
                f.write("    PyTextual,\n")
                f.write("    PyNumerical,\n")
                f.write("    PyDictionary,\n")
                f.write("    PyNamedObjectContainer,\n")
                f.write("    PyCommand,\n")
                f.write("    PyQuery\n")
                f.write(")\n\n\n")
                api_tree_val = {
                    name: self._write_static_info("Root", info.static_info, f)
                }
                for mode in info.modes:
                    if mode in ("solver", "meshing"):
                        key = f"<{mode}_session>"
                        api_tree[key].update(api_tree_val)
        return api_tree

    def _delete_generated_files(self):
        for _, info in self._static_info.items():
            if info.file_name.exists():
                info.file_name.unlink()
        if Path(_MESHING_DM_DOC_DIR).exists():
            shutil.rmtree(Path(_MESHING_DM_DOC_DIR))
        if Path(_SOLVER_DM_DOC_DIR).exists():
            shutil.rmtree(Path(_SOLVER_DM_DOC_DIR))


def generate(version, static_infos: dict, verbose: bool = False):
    """Generate datamodel API classes."""
    return DataModelGenerator(version, static_infos, verbose).write_static_info()


if __name__ == "__main__":
    solver = launch_fluent()
    meshing = launch_fluent(mode=FluentMode.MESHING)
    version = get_version_for_file_name(session=solver)
    static_infos = {
        StaticInfoType.DATAMODEL_WORKFLOW: meshing._datamodel_service_se.get_static_info(
            "workflow"
        ),
        StaticInfoType.DATAMODEL_MESHING: meshing._datamodel_service_se.get_static_info(
            "meshing"
        ),
        StaticInfoType.DATAMODEL_PART_MANAGEMENT: meshing._datamodel_service_se.get_static_info(
            "PartManagement"
        ),
        StaticInfoType.DATAMODEL_PM_FILE_MANAGEMENT: meshing._datamodel_service_se.get_static_info(
            "PMFileManagement"
        ),
        StaticInfoType.DATAMODEL_PREFERENCES: solver._datamodel_service_se.get_static_info(
            "preferences"
        ),
    }
    if FluentVersion(version) >= FluentVersion.v231:
        flicing = launch_fluent(mode=FluentMode.SOLVER_ICING)
        static_infos[StaticInfoType.DATAMODEL_FLICING] = (
            flicing._datamodel_service_se.get_static_info("flserver")
        )
        static_infos[StaticInfoType.DATAMODEL_SOLVER_WORKFLOW] = (
            solver._datamodel_service_se.get_static_info("solverworkflow")
        )
    if FluentVersion(version) >= FluentVersion.v242:
        static_infos[StaticInfoType.DATAMODEL_MESHING_UTILITIES] = (
            meshing._datamodel_service_se.get_static_info("MeshingUtilities")
        )
    parser = argparse.ArgumentParser(
        description="A script to write Fluent API files with an optional verbose output."
    )
    parser.add_argument(
        "-v",
        "--verbose",
        action="store_true",
        help="Show paths of written Fluent API files.",
    )
    args = parser.parse_args()
    generate(version, static_infos, args.verbose)



# Copyright (C) 2021 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Provides a module to get base Meshing session."""

import logging

import ansys.fluent.core as pyfluent
from ansys.fluent.core.fluent_connection import FluentConnection
from ansys.fluent.core.meshing.meshing_workflow import (
    CreateWorkflow,
    LoadWorkflow,
    WorkflowMode,
    name_to_identifier_map,
)
from ansys.fluent.core.services.datamodel_se import PyMenuGeneric
from ansys.fluent.core.session_shared import (
    _CODEGEN_MSG_DATAMODEL,
    _make_datamodel_module,
    _make_tui_module,
)
from ansys.fluent.core.utils.fluent_version import (
    FluentVersion,
    get_version_for_file_name,
)

pyfluent_logger = logging.getLogger("pyfluent.general")
datamodel_logger = logging.getLogger("pyfluent.datamodel")


class BaseMeshing:
    """Encapsulates base methods of a meshing session."""

    def __init__(
        self,
        session_execute_tui,
        fluent_connection: FluentConnection,
        fluent_version,
        datamodel_service_tui,
        datamodel_service_se,
    ):
        """BaseMeshing session.

        Parameters
        ----------
        session_execute_tui (_type_):
            Executes Fluent’s SolverTUI methods.
        fluent_connection (:ref:`ref_fluent_connection`):
            Encapsulates a Fluent connection.
        """
        self._tui_service = datamodel_service_tui
        self._se_service = datamodel_service_se
        self._fluent_connection = fluent_connection
        self._tui = None
        self._meshing = None
        self._fluent_version = fluent_version
        self._meshing_utilities = None
        self._old_workflow = None
        self._part_management = None
        self._pm_file_management = None
        self._preferences = None
        self._session_execute_tui = session_execute_tui
        self._product_version = None
        self._current_workflow = None

    def get_fluent_version(self) -> FluentVersion:
        """Gets and returns the fluent version."""
        pyfluent_logger.debug("Fluent version = " + str(self._fluent_version))
        return FluentVersion(self._fluent_version)

    @property
    def _version(self):
        """Fluent's product version."""
        if self._product_version is None:
            self._product_version = get_version_for_file_name(session=self)
        return self._product_version

    @property
    def tui(self):
        """Instance of ``main_menu`` on which Fluent's SolverTUI methods can be
        executed."""
        if self._tui is None:
            self._tui = _make_tui_module(self, "meshing")

        return self._tui

    @property
    def meshing(self):
        """Meshing object."""
        if self._meshing is None:
            self._meshing = _make_datamodel_module(self, "meshing")
        return self._meshing

    @property
    def _meshing_utilities_root(self):
        """Datamodel root of meshing_utilities."""
        try:
            if self.get_fluent_version() >= FluentVersion.v242:
                from ansys.fluent.core import CODEGEN_OUTDIR

                meshing_utilities_module = pyfluent.utils.load_module(
                    f"meshing_utilities_{self._version}",
                    CODEGEN_OUTDIR
                    / f"datamodel_{self._version}"
                    / "meshing_utilities.py",
                )
                meshing_utilities_root = meshing_utilities_module.Root(
                    self._se_service, "MeshingUtilities", []
                )
        except (ImportError, FileNotFoundError):
            datamodel_logger.warning(_CODEGEN_MSG_DATAMODEL)
            if self.get_fluent_version() >= FluentVersion.v242:
                meshing_utilities_root = PyMenuGeneric(
                    self._se_service, "MeshingUtilities"
                )
        return meshing_utilities_root

    @property
    def meshing_utilities(self):
        """A wrapper over the Fluent's meshing queries."""
        if self._meshing_utilities is None:
            self._meshing_utilities = self._meshing_utilities_root
        return self._meshing_utilities

    @property
    def workflow(self):
        """Datamodel root of workflow."""
        if self._old_workflow is None:
            self._old_workflow = _make_datamodel_module(self, "workflow")
        return self._old_workflow

    def watertight_workflow(self, initialize: bool = True):
        """Datamodel root of workflow."""
        self._current_workflow = WorkflowMode.WATERTIGHT_MESHING_MODE.value(
            _make_datamodel_module(self, "workflow"),
            self.meshing,
            self.get_fluent_version(),
            initialize,
        )
        return self._current_workflow

    def fault_tolerant_workflow(self, initialize: bool = True):
        """Datamodel root of workflow."""
        self._current_workflow = WorkflowMode.FAULT_TOLERANT_MESHING_MODE.value(
            _make_datamodel_module(self, "workflow"),
            self.meshing,
            self.PartManagement,
            self.PMFileManagement,
            self.get_fluent_version(),
            initialize,
        )
        return self._current_workflow

    def two_dimensional_meshing_workflow(self, initialize: bool = True):
        """Data model root of the workflow."""
        self._current_workflow = WorkflowMode.TWO_DIMENSIONAL_MESHING_MODE.value(
            _make_datamodel_module(self, "workflow"),
            self.meshing,
            self.get_fluent_version(),
            initialize,
        )
        return self._current_workflow

    def topology_based_meshing_workflow(self, initialize: bool = True):
        """Datamodel root of workflow."""
        self._current_workflow = WorkflowMode.TOPOLOGY_BASED_MESHING_MODE.value(
            _make_datamodel_module(self, "workflow"),
            self.meshing,
            self.get_fluent_version(),
            initialize,
        )
        return self._current_workflow

    def load_workflow(self, file_path: str):
        """Datamodel root of workflow."""
        self._current_workflow = LoadWorkflow(
            _make_datamodel_module(self, "workflow"),
            self.meshing,
            file_path,
            self.get_fluent_version(),
        )
        return self._current_workflow

    def create_workflow(self, initialize: bool = True):
        """Datamodel root of the workflow."""
        self._current_workflow = CreateWorkflow(
            _make_datamodel_module(self, "workflow"),
            self.meshing,
            self.get_fluent_version(),
            initialize,
        )
        return self._current_workflow

    def _check_workflow_type(self, name: str):
        return getattr(self.meshing.GlobalSettings, name_to_identifier_map[name])()

    def _get_current_workflow(self, name: str):
        if self._current_workflow and self._current_workflow._name == name:
            return self._current_workflow

    @property
    def current_workflow(self):
        """Datamodel root of the workflow.

        Raises
        ------
        RuntimeError
            If no workflow is initialized.
        """
        if self._check_workflow_type(
            "Watertight Geometry"
        ) and self._check_workflow_type("Fault-tolerant Meshing"):
            raise RuntimeError("No workflow initialized.")
        elif self._check_workflow_type("Watertight Geometry"):
            return self._get_current_workflow(
                "Watertight Geometry"
            ) or self.watertight_workflow(initialize=False)
        elif self._check_workflow_type("Fault-tolerant Meshing"):
            return self._get_current_workflow(
                "Fault-tolerant Meshing"
            ) or self.fault_tolerant_workflow(initialize=False)
        elif self._check_workflow_type("2D Meshing"):
            return self._get_current_workflow(
                "2D Meshing"
            ) or self.two_dimensional_meshing_workflow(initialize=False)
        elif self._check_workflow_type("Topology Based Meshing"):
            return self._get_current_workflow(
                "Topology Based Meshing"
            ) or self.topology_based_meshing_workflow(initialize=False)
        else:
            return self.create_workflow(initialize=False)

    @property
    def PartManagement(self):
        """Datamodel root of ``PartManagement``."""
        if self._part_management is None:
            self._part_management = _make_datamodel_module(self, "PartManagement")
        return self._part_management

    @property
    def PMFileManagement(self):
        """Datamodel root of PMFileManagement."""
        if self._pm_file_management is None:
            self._pm_file_management = _make_datamodel_module(self, "PMFileManagement")
        return self._pm_file_management

    @property
    def preferences(self):
        """Datamodel root of preferences."""
        if self._preferences is None:
            self._preferences = _make_datamodel_module(self, "preferences")
        return self._preferences



# Copyright (C) 2021 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Module for analyzing API docstrings."""

# N.b. misspellings:
# - selected missing last e.
# - many known API misspellings also have corresponding
#   misspellingling for docstring.
# - CGSN

import re

_choose_words = ("Choose", "Indicate", "Select", "Specify")

_remove_first_words = _choose_words + ("Enter", "Set", "To")

# Define problematic patterns with exclusions
_dubious_patterns = [
    (r"Create an instance of this.", ()),
    (r"Enter the adaption .* menu.", ()),
    (r"Allows you to .*", ()),
    (r"Select to .*", ("method",)),
    (r"Choose .*", ("method",)),
    (r"Enter .*", ("method",)),
    (r"Indicate .*", ("method",)),
    (r"Perform .*", ("method",)),
    (r"Select .*", ("method",)),
    (r"Set .*", ("method",)),
    (r"Specify .*", ("method",)),
    (r".* menu.", ()),
    (r"To .*", ()),
    (r"Name for an object.*", ()),
]

# Rewriting problematic patterns for suggestions

_bad_0 = {
    r"Create an instance of this.": r"Create a new instance of the current object type.",
    r"Allows you to (.*)": r"\1",
}

_bad_1 = {
    rf"{w} (whether or not|whether|if) (.*)": r"Specifies whether \2"
    for w in _choose_words
}

_bad_2 = {
    r"Select to (.*)": r"Specifies whether to \1",
    r"Perform (.*)": r"Specifies whether to perform \1",
    r"Enter a (.*)": r"The \1",
    r"Enter an (.*)": r"The \1",
    r"(.*) menu.": r"\1 object.",
}

_bad_3 = {rf"{w} (.*)": r"\1" for w in _remove_first_words}

_bad_4 = {r"Name for an object(.*)": r"Object name\1"}

_bad_to_suggested = dict(**_bad_0, **_bad_1, **_bad_2, **_bad_3, **_bad_4)

# Pre-compile fixed-format empty patterns (case-sensitive)
_empty_patterns = [
    re.compile(f"'.*' {item}.") for item in ("child", "command", "query")
]


def _suggest_fix(docstring: str):
    """Suggest an improved version of a problematic docstring."""
    for bad, suggestion in _bad_to_suggested.items():
        match = re.match(bad, docstring)
        if match:
            replacement = match.expand(suggestion)

            # Capitalize if original docstring started with a capital letter
            if docstring[0].isupper():
                replacement = replacement[0].upper() + replacement[1:]

            # Ensure we don't add duplicate periods
            if replacement.endswith("."):
                return replacement
            return replacement + "."

    return None  # No fix available


def _fixed_doc_string(api_item_type, docstring):
    for pattern_str, exclude_types in _dubious_patterns:
        if api_item_type in exclude_types:
            continue

        if re.match(pattern_str, docstring):
            fix = _suggest_fix(docstring)
            new_fix = None
            if fix:
                new_fix = _fixed_doc_string(api_item_type, fix)
            return new_fix or fix

    return None


class _DocStringAnalysis:
    """Analyzes docstrings for potential issues and categorizes them."""

    def __init__(self):
        """Initialize storage for categorized docstring issues."""
        self.dubious = {}
        self.empty = {}
        self.clean = {}

    def analyse(self, api_path: str, api_item_type: str, api_cls: object):
        """Extract and analyze the docstring for a given API item."""
        docstring = self.get_basic_docstring(api_cls)
        self.analyse_docstring(api_path, api_item_type, docstring)

    def analyse_docstring(self, api_path: str, api_item_type: str, docstring: str):
        """Analyze a docstring and categorize it as clean, dubious, or empty."""

        # Check for empty docstrings first (case-sensitive)
        for pattern in _empty_patterns:
            if pattern.match(docstring):
                self.empty[api_path] = (api_item_type, docstring)
                return

        # Check for dubious patterns
        # TODO just use None?
        fix = _fixed_doc_string(api_item_type, docstring)
        if fix:
            self.dubious[api_path] = (api_item_type, docstring, fix)
            return

        # If no issues, classify as clean
        self.clean[api_path] = (api_item_type, docstring)

    def get_basic_docstring(self, api_cls: object):
        """Extract and clean a class docstring, handling None values."""
        raw_docstring = api_cls.__doc__ or ""
        docstring = "".join(raw_docstring.splitlines()).strip()

        # Remove parameters section if present
        params_section = "        Parameters    ----------        "
        idx = docstring.find(params_section)
        return docstring[:idx] if idx != -1 else docstring

    def show_results(self):
        """Print a categorized summary of analyzed docstrings."""
        print("\nSummary:\n")
        for label, result in [
            ("Empty", self.empty),
            ("Dubious", self.dubious),
            ("Clean", self.clean),
        ]:
            count = len(result)
            print(f"{label}: {count}")
            if label == "Dubious":
                fix_count = len([i for i in result.values() if i[2]])
                print(
                    f"\tNumber with/without suggested fixes: {fix_count}/{count - fix_count}"
                )

        print("\nDetailed results:")
        for label, result in [
            ("Empty", self.empty),
            ("Dubious", self.dubious),
            ("Clean", self.clean),
        ]:
            print(f"\n{label} ({len(result)}):")
            for k, (t, s, *extra) in result.items():
                print(f"{k} ({t}):")
                print(f"\t{s}")

                # Suggest fixes for dubious patterns
                if label == "Dubious" and extra:
                    suggested = extra[0]
                    if suggested:
                        print(f"\tSuggested Fix: {suggested}")
                    else:
                        print("\tNo suggested fix")


def show_all_doc_strings(api_path: str, api_item_type: str, api_cls: object):
    """Prints all docstrings for debugging."""
    indent = "  " * (api_path.count("."))
    bullet = f"{indent}-"

    print(f"{bullet} item: {api_path.split('.')[-1]}")
    print(f"{bullet} type: {api_item_type}")
    print(f"{bullet} docstring: {api_cls.__doc__}")


if __name__ == "__main__":
    import argparse
    import importlib

    from ansys.fluent.core.codegen import walk_api

    # Sample Usage
    #
    # Whole Fluent settings API check with default settings
    # >>> python src\ansys\fluent\core\solver\_docstrings.py --api-version=252

    parser = argparse.ArgumentParser()
    parser.add_argument("--cls", dest="api_cls", type=str, default="root")
    parser.add_argument("--path", dest="api_path", type=str, default="")
    parser.add_argument("--output-type", dest="output_type", type=str, default="stats")
    parser.add_argument("--api-version", dest="api_version", type=str, default="")

    args = parser.parse_args()

    api_cls = args.api_cls
    api_path = args.api_path
    output_type = args.output_type
    api_version = args.api_version

    mod = importlib.import_module(
        name=f"ansys.fluent.core.generated.solver.settings_{api_version}"
    )
    cls = getattr(mod, api_cls)

    if output_type == "show":
        walk_api.walk_api(cls, show_all_doc_strings, api_path)
    elif output_type == "stats":
        analysis = _DocStringAnalysis()
        walk_api.walk_api(cls, analysis.analyse, api_path)
        analysis.show_results()



"""Helper module to generate Fluent API classes."""

import argparse
from time import time

from ansys.fluent.core import CODEGEN_OUTDIR, FluentMode, FluentVersion, launch_fluent
from ansys.fluent.core.codegen import StaticInfoType, allapigen
from ansys.fluent.core.codegen.print_fluent_version import print_fluent_version
from ansys.fluent.core.search import _generate_api_data
from ansys.fluent.core.utils.fluent_version import get_version_for_file_name

if __name__ == "__main__":
    t0 = time()
    meshing = launch_fluent(mode=FluentMode.MESHING)
    version = get_version_for_file_name(session=meshing)
    gt_222 = FluentVersion(version) > FluentVersion.v222
    ge_231 = FluentVersion(version) >= FluentVersion.v231
    ge_242 = FluentVersion(version) >= FluentVersion.v242

    static_infos = {
        StaticInfoType.DATAMODEL_WORKFLOW: meshing._datamodel_service_se.get_static_info(
            "workflow"
        ),
        StaticInfoType.DATAMODEL_MESHING: meshing._datamodel_service_se.get_static_info(
            "meshing"
        ),
        StaticInfoType.DATAMODEL_PART_MANAGEMENT: meshing._datamodel_service_se.get_static_info(
            "PartManagement"
        ),
        StaticInfoType.DATAMODEL_PM_FILE_MANAGEMENT: meshing._datamodel_service_se.get_static_info(
            "PMFileManagement"
        ),
    }
    if gt_222:
        static_infos[StaticInfoType.TUI_MESHING] = (
            meshing._datamodel_service_tui.get_static_info("")
        )
    if ge_242:
        static_infos[StaticInfoType.DATAMODEL_MESHING_UTILITIES] = (
            meshing._datamodel_service_se.get_static_info("MeshingUtilities")
        )
    meshing.exit()

    solver = launch_fluent(
        mode=FluentMode.SOLVER_ICING if ge_231 else FluentMode.SOLVER
    )
    static_infos[StaticInfoType.DATAMODEL_PREFERENCES] = (
        solver._datamodel_service_se.get_static_info("preferences")
    )
    static_infos[StaticInfoType.SETTINGS] = solver._settings_service.get_static_info()
    if gt_222:
        static_infos[StaticInfoType.TUI_SOLVER] = (
            solver._datamodel_service_tui.get_static_info("")
        )
    if ge_231:
        static_infos[StaticInfoType.DATAMODEL_FLICING] = (
            solver._datamodel_service_se.get_static_info("flserver")
        )
        static_infos[StaticInfoType.DATAMODEL_SOLVER_WORKFLOW] = (
            solver._datamodel_service_se.get_static_info("solverworkflow")
        )
    t1 = time()
    print(f"\nTime to fetch static info: {t1 - t0:.2f} seconds")
    CODEGEN_OUTDIR.mkdir(parents=True, exist_ok=True)
    print_fluent_version(solver._app_utilities)
    solver.exit()
    parser = argparse.ArgumentParser(
        description="A script to write Fluent API files with an optional verbose output."
    )
    parser.add_argument(
        "-v",
        "--verbose",
        action="store_true",
        help="Show paths of written Fluent API files.",
    )
    args = parser.parse_args()
    allapigen.generate(version, static_infos, args.verbose)
    t2 = time()
    print(f"Time to generate APIs: {t2 - t1:.2f} seconds")
    _generate_api_data(version=version)



# Copyright (C) 2021 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Provides a module for the event loop."""

import asyncio
from functools import partial

loop = asyncio.get_event_loop()


def execute_in_event_loop_threadsafe(f):
    """Decorates function to be executed in an event loop from another thread."""

    def cb(*args, **kwargs):
        par = partial(f, *args, **kwargs)
        loop.call_soon_threadsafe(par)

    return cb


def execute_in_event_loop(f):
    """Decorates function to be executed in an event loop."""

    def cb(*args, **kwargs):
        par = partial(f, *args, **kwargs)
        loop.call_soon(par)

    return cb



# Copyright (C) 2021 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import os
from pathlib import Path
import platform
import tempfile
from tempfile import TemporaryDirectory

import pytest

import ansys.fluent.core as pyfluent
from ansys.fluent.core import PyFluentDeprecationWarning
from ansys.fluent.core.examples.downloads import download_file
from ansys.fluent.core.exceptions import DisallowedValuesError, InvalidArgument
from ansys.fluent.core.launcher import launcher_utils
from ansys.fluent.core.launcher.error_handler import (
    GPUSolverSupportError,
    InvalidIpPort,
    LaunchFluentError,
    _raise_non_gui_exception_in_windows,
)
from ansys.fluent.core.launcher.launcher import create_launcher
from ansys.fluent.core.launcher.launcher_utils import (
    _build_journal_argument,
    is_windows,
)
from ansys.fluent.core.launcher.process_launch_string import (
    _build_fluent_launch_args_string,
    get_fluent_exe_path,
)
from ansys.fluent.core.launcher.pyfluent_enums import (
    FluentLinuxGraphicsDriver,
    FluentMode,
    FluentWindowsGraphicsDriver,
    LaunchMode,
    UIMode,
)
from ansys.fluent.core.utils.fluent_version import AnsysVersionNotFound, FluentVersion
import ansys.platform.instancemanagement as pypim


def test_gpu_version_error():
    with pytest.raises(GPUSolverSupportError):
        pyfluent.launch_fluent(
            mode="meshing",
            dimension=2,
            precision="single",
            processor_count=5,
            ui_mode="gui",
            gpu=True,
        )
        pyfluent.setup_for_fluent(
            mode="meshing",
            dimension=2,
            precision="single",
            processor_count=5,
            ui_mode="gui",
            gpu=True,
        )


def test_mode():
    with pytest.raises(DisallowedValuesError):
        pyfluent.launch_fluent(
            mode="meshing-solver",
            start_container=False,
        )


@pytest.mark.standalone
def test_unsuccessful_fluent_connection():
    # start-timeout is intentionally provided to be 2s for the connection to fail
    with pytest.raises(LaunchFluentError) as ex:
        pyfluent.launch_fluent(mode="solver", start_timeout=2)
    # TimeoutError -> LaunchFluentError
    assert isinstance(ex.value.__context__, TimeoutError)


@pytest.mark.fluent_version("<24.1")
def test_non_gui_in_windows_throws_exception():
    default_windows_flag = launcher_utils.is_windows()
    launcher_utils.is_windows = lambda: True
    try:
        with pytest.raises(InvalidArgument):
            _raise_non_gui_exception_in_windows(UIMode.NO_GUI, FluentVersion.v232)
        with pytest.raises(InvalidArgument):
            _raise_non_gui_exception_in_windows(UIMode.NO_GUI, FluentVersion.v231)
        with pytest.raises(InvalidArgument):
            _raise_non_gui_exception_in_windows(UIMode.NO_GUI, FluentVersion.v222)
        with pytest.raises(InvalidArgument):
            _raise_non_gui_exception_in_windows(
                UIMode.NO_GUI_OR_GRAPHICS, FluentVersion.v232
            )
        with pytest.raises(InvalidArgument):
            _raise_non_gui_exception_in_windows(
                UIMode.NO_GUI_OR_GRAPHICS, FluentVersion.v231
            )
        with pytest.raises(InvalidArgument):
            _raise_non_gui_exception_in_windows(
                UIMode.NO_GUI_OR_GRAPHICS, FluentVersion.v222
            )
    finally:
        launcher_utils.is_windows = lambda: default_windows_flag


@pytest.mark.fluent_version(">=24.1")
def test_non_gui_in_windows_does_not_throw_exception():
    default_windows_flag = launcher_utils.is_windows()
    launcher_utils.is_windows = lambda: True
    try:
        _raise_non_gui_exception_in_windows(UIMode.NO_GUI, FluentVersion.v241)
        _raise_non_gui_exception_in_windows(
            UIMode.NO_GUI_OR_GRAPHICS, FluentVersion.v241
        )
        _raise_non_gui_exception_in_windows(UIMode.NO_GUI, FluentVersion.v242)
        _raise_non_gui_exception_in_windows(
            UIMode.NO_GUI_OR_GRAPHICS, FluentVersion.v242
        )
    finally:
        launcher_utils.is_windows = lambda: default_windows_flag


def test_container_launcher():
    # test dry_run
    container_dict = pyfluent.launch_fluent(start_container=True, dry_run=True)
    assert isinstance(container_dict, dict)
    assert len(container_dict) > 1

    # test run with configuration dict
    session = pyfluent.launch_fluent(container_dict=container_dict)
    assert session.health_check.is_serving


@pytest.mark.standalone
def test_case_load():
    # Test that launch_fluent() works with a case file as an argument
    case_name = download_file(
        "mixing_elbow.cas.h5",
        "pyfluent/mixing_elbow",
    )
    session = pyfluent.launch_fluent(case_file_name=case_name)

    # Case loaded
    assert session.setup.boundary_conditions.is_active()
    # Mesh available because not lightweight
    if not session.get_fluent_version() < FluentVersion.v231:
        assert session.mesh.quality.is_active()
    # Data not loaded
    assert not session.fields.field_data.is_data_valid()

    session.exit()


@pytest.mark.standalone
@pytest.mark.fluent_version(">=23.2")
def test_case_lightweight_setup():
    # Test that launch_fluent() correctly performs lightweight setup
    case_name = download_file(
        "mixing_elbow.cas.h5",
        "pyfluent/mixing_elbow",
    )
    session = pyfluent.launch_fluent(
        case_file_name=case_name,
        lightweight_mode=True,
    )

    # Case loaded
    assert session.setup.boundary_conditions.is_active()
    # Mesh not available because lightweight
    assert not session.mesh.quality.is_active()
    # Data not loaded
    assert not session.fields.field_data.is_data_valid()


@pytest.mark.standalone
def test_case_data_load():
    # Test that launch_fluent() works with a case+data file as an argument
    case_name = download_file(
        "mixing_elbow.cas.h5",
        "pyfluent/mixing_elbow",
    )
    download_file(
        "mixing_elbow.dat.h5",
        "pyfluent/mixing_elbow",
    )
    session = pyfluent.launch_fluent(case_data_file_name=case_name)

    # Case loaded
    assert session.setup.boundary_conditions.is_active()
    # Mesh available because not lightweight
    if not session.get_fluent_version() < FluentVersion.v231:
        assert session.mesh.quality.is_active()
    # Data loaded
    assert session.fields.field_data.is_data_valid()

    session.exit()


def test_gpu_launch_arg():
    assert (
        _build_fluent_launch_args_string(
            gpu=True, additional_arguments="", processor_count=None
        ).strip()
        == "3ddp -gpu -hidden"
        if is_windows()
        else "3ddp -gpu -gu"
    )
    assert (
        _build_fluent_launch_args_string(
            gpu=[1, 2, 4], additional_arguments="", processor_count=None
        ).strip()
        == "3ddp -gpu=1,2,4 -hidden"
        if is_windows()
        else "3ddp -gpu=1,2,4 -gu"
    )


def test_gpu_launch_arg_additional_arg():
    assert (
        _build_fluent_launch_args_string(
            additional_arguments="-gpu", processor_count=None
        ).strip()
        == "3ddp -gpu -hidden"
        if is_windows()
        else "3ddp -gpu -gu"
    )
    assert (
        _build_fluent_launch_args_string(
            additional_arguments="-gpu=1,2,4", processor_count=None
        ).strip()
        == "3ddp -gpu=1,2,4 -hidden"
        if is_windows()
        else "3ddp -gpu=1,2,4 -gu"
    )


def test_get_fluent_exe_path_when_nothing_is_set(helpers):
    helpers.delete_all_awp_vars()
    with pytest.raises(AnsysVersionNotFound):
        get_fluent_exe_path()
    with pytest.raises(AnsysVersionNotFound):
        FluentVersion.get_latest_installed()


def test_get_fluent_exe_path_from_awp_root_222(helpers):
    helpers.mock_awp_vars(version="222")
    if platform.system() == "Windows":
        expected_path = Path("ansys_inc/v222/fluent") / "ntbin" / "win64" / "fluent.exe"
    else:
        expected_path = Path("ansys_inc/v222/fluent") / "bin" / "fluent"
    assert FluentVersion.get_latest_installed() == FluentVersion.v222
    assert get_fluent_exe_path() == expected_path


def test_get_fluent_exe_path_from_awp_root_231(helpers):
    helpers.mock_awp_vars(version="231")
    if platform.system() == "Windows":
        expected_path = Path("ansys_inc/v231/fluent") / "ntbin" / "win64" / "fluent.exe"
    else:
        expected_path = Path("ansys_inc/v231/fluent") / "bin" / "fluent"
    assert FluentVersion.get_latest_installed() == FluentVersion.v231
    assert get_fluent_exe_path() == expected_path


def test_get_fluent_exe_path_from_awp_root_232(helpers):
    helpers.mock_awp_vars(version="232")
    if platform.system() == "Windows":
        expected_path = Path("ansys_inc/v232/fluent") / "ntbin" / "win64" / "fluent.exe"
    else:
        expected_path = Path("ansys_inc/v232/fluent") / "bin" / "fluent"
    assert FluentVersion.get_latest_installed() == FluentVersion.v232
    assert get_fluent_exe_path() == expected_path


def test_get_fluent_exe_path_from_awp_root_241(helpers):
    helpers.mock_awp_vars(version="241")
    if platform.system() == "Windows":
        expected_path = Path("ansys_inc/v241/fluent") / "ntbin" / "win64" / "fluent.exe"
    else:
        expected_path = Path("ansys_inc/v241/fluent") / "bin" / "fluent"
    assert FluentVersion.get_latest_installed() == FluentVersion.v241
    assert get_fluent_exe_path() == expected_path


def test_get_fluent_exe_path_from_product_version_launcher_arg(helpers):
    helpers.mock_awp_vars()
    if platform.system() == "Windows":
        expected_path = Path("ansys_inc/v231/fluent") / "ntbin" / "win64" / "fluent.exe"
    else:
        expected_path = Path("ansys_inc/v231/fluent") / "bin" / "fluent"
    assert get_fluent_exe_path(product_version=231) == expected_path


def test_get_fluent_exe_path_from_pyfluent_fluent_root(helpers, monkeypatch):
    helpers.mock_awp_vars()
    monkeypatch.setenv("PYFLUENT_FLUENT_ROOT", "dev/vNNN/fluent")
    if platform.system() == "Windows":
        expected_path = Path("dev/vNNN/fluent") / "ntbin" / "win64" / "fluent.exe"
    else:
        expected_path = Path("dev/vNNN/fluent") / "bin" / "fluent"
    assert get_fluent_exe_path() == expected_path


def test_watchdog_launch(monkeypatch):
    monkeypatch.setenv("PYFLUENT_WATCHDOG_EXCEPTION_ON_ERROR", "1")
    pyfluent.launch_fluent(start_watchdog=True)


@pytest.mark.standalone
def test_create_standalone_launcher():
    kwargs = dict(
        ui_mode=UIMode.NO_GUI,
        graphics_driver=(
            FluentWindowsGraphicsDriver.AUTO
            if is_windows()
            else FluentLinuxGraphicsDriver.AUTO
        ),
        env={},
    )

    standalone_meshing_launcher = create_launcher(
        LaunchMode.STANDALONE, mode=FluentMode.MESHING, **kwargs
    )
    standalone_meshing_session = standalone_meshing_launcher()
    assert standalone_meshing_session
    standalone_meshing_session.exit()

    standalone_solver_launcher = create_launcher(
        LaunchMode.STANDALONE, mode=FluentMode.SOLVER, **kwargs
    )
    standalone_solver_session = standalone_solver_launcher()
    assert standalone_solver_session
    standalone_solver_session.exit()


def test_fluent_launchers():
    kwargs = dict(
        ui_mode=UIMode.NO_GUI,
        graphics_driver=(
            FluentWindowsGraphicsDriver.AUTO
            if is_windows()
            else FluentLinuxGraphicsDriver.AUTO
        ),
    )
    kargs = dict(
        ui_mode=kwargs["ui_mode"],
        graphics_driver=kwargs["graphics_driver"],
        product_version=None,
        precision=None,
        processor_count=None,
        start_timeout=None,
        additional_arguments="",
        container_dict=None,
        dry_run=None,
        cleanup_on_exit=None,
        start_transcript=None,
        py=None,
        gpu=None,
        start_watchdog=None,
        file_transfer_service=None,
    )
    container_meshing_launcher = create_launcher(
        LaunchMode.CONTAINER,
        mode=FluentMode.MESHING,
        **kargs,
    )
    container_meshing_session = container_meshing_launcher()
    assert container_meshing_session
    container_meshing_session.exit()

    container_solver_launcher = create_launcher(
        LaunchMode.CONTAINER,
        mode=FluentMode.SOLVER,
        **kargs,
    )
    container_solver_session = container_solver_launcher()
    assert container_solver_session
    container_solver_session.exit()

    if pypim.is_configured():
        pim_meshing_launcher = create_launcher(
            LaunchMode.PIM, mode=FluentMode.MESHING, **kwargs
        )
        pim_meshing_session = pim_meshing_launcher()
        assert pim_meshing_session
        pim_meshing_session.exit()

        pim_solver_launcher = create_launcher(
            LaunchMode.PIM, mode=FluentMode.SOLVER, **kwargs
        )
        pim_solver_session = pim_solver_launcher()
        assert pim_solver_session
        pim_solver_session.exit()


@pytest.mark.parametrize(
    "topy,journal_file_names,result,raises",
    [
        (None, "a.jou", ' -i "a.jou"', pytest.wont_raise()),
        (None, ["a.jou", "b.jou"], ' -i "a.jou" -i "b.jou"', pytest.wont_raise()),
        (True, "a.jou", ' -i "a.jou" -topy', pytest.wont_raise()),
        (True, ["a.jou", "b.jou"], ' -i "a.jou" -i "b.jou" -topy', pytest.wont_raise()),
        ("c.py", "a.jou", ' -i "a.jou" -topy="c.py"', pytest.wont_raise()),
        (
            "c.py",
            ["a.jou", "b.jou"],
            ' -i "a.jou" -i "b.jou" -topy="c.py"',
            pytest.wont_raise(),
        ),
        (None, 5, None, pytest.raises(TypeError)),
        (True, None, None, pytest.raises(InvalidArgument)),
    ],
)
def test_build_journal_argument(topy, journal_file_names, result, raises):
    with raises:
        assert _build_journal_argument(topy, journal_file_names) == result


def test_show_gui_raises_warning():
    with pytest.warns(PyFluentDeprecationWarning):
        solver = pyfluent.launch_fluent(show_gui=True)
        solver.exit()


def test_fluent_enums():
    assert UIMode.GUI.value == "gui"
    assert UIMode("gui") == UIMode.GUI
    with pytest.raises(ValueError):
        UIMode("")
    with pytest.raises(TypeError):
        assert UIMode.NO_GUI < FluentWindowsGraphicsDriver.AUTO


def test_exposure_and_graphics_driver_arguments():
    with pytest.raises(ValueError):
        pyfluent.launch_fluent(ui_mode="gu")
    with pytest.raises(ValueError):
        pyfluent.launch_fluent(graphics_driver="x11" if is_windows() else "dx11")
    for m in UIMode:
        string1 = _build_fluent_launch_args_string(
            ui_mode=m, additional_arguments="", processor_count=None
        ).strip()
        string2 = (
            f"3ddp -{m.get_fluent_value()[0]}" if m.get_fluent_value()[0] else "3ddp"
        )
        assert string1 == string2
    for e in (FluentWindowsGraphicsDriver, FluentLinuxGraphicsDriver):
        for m in e:
            msg = _build_fluent_launch_args_string(
                graphics_driver=m, additional_arguments="", processor_count=None
            ).strip()
            if is_windows():
                assert (
                    msg == f"3ddp -hidden -driver {m.get_fluent_value()[0]}"
                    if m.get_fluent_value()[0]
                    else " 3ddp -hidden"
                )
            else:
                assert (
                    msg == f"3ddp -gu -driver {m.get_fluent_value()[0]}"
                    if m.get_fluent_value()[0]
                    else " 3ddp -gu"
                )


def test_additional_arguments_fluent_launch_args_string():
    additional_arguments = "-ws -ws-port=5000 -i test.jou"
    assert additional_arguments in _build_fluent_launch_args_string(
        additional_arguments=additional_arguments,
        processor_count=4,
    )


def test_processor_count():
    def get_processor_count(solver):
        return int(solver.rp_vars("parallel/nprocs_string").strip('"'))

    with pyfluent.launch_fluent(processor_count=2) as solver:
        assert get_processor_count(solver) == 2
    # The following check is not yet supported for container launch
    # https://github.com/ansys/pyfluent/issues/2624
    # with pyfluent.launch_fluent(additional_arguments="-t2") as solver:
    #     assert get_processor_count(solver) == 2


def test_container_warning_for_mount_source(caplog):
    container_dict = {
        "mount_source": os.getcwd(),
        "mount_target": "/mnt/pyfluent/tests",
    }
    _ = pyfluent.launch_fluent(container_dict=container_dict)
    assert container_dict["mount_source"] in caplog.text
    assert container_dict["mount_target"] in caplog.text


# runs only in container till cwd is supported for container launch
def test_fluent_automatic_transcript(monkeypatch):
    with monkeypatch.context() as m:
        m.setattr(pyfluent, "FLUENT_AUTOMATIC_TRANSCRIPT", True)
        with TemporaryDirectory(dir=pyfluent.EXAMPLES_PATH) as tmp_dir:
            with pyfluent.launch_fluent(container_dict=dict(working_dir=tmp_dir)):
                assert list(Path(tmp_dir).glob("*.trn"))
    with TemporaryDirectory(dir=pyfluent.EXAMPLES_PATH) as tmp_dir:
        with pyfluent.launch_fluent(container_dict=dict(working_dir=tmp_dir)):
            assert not list(Path(tmp_dir).glob("*.trn"))


def test_standalone_launcher_dry_run(monkeypatch):
    monkeypatch.setenv("PYFLUENT_LAUNCH_CONTAINER", "0")
    fluent_path = r"\x\y\z\fluent.exe"
    fluent_launch_string, server_info_file_name = pyfluent.launch_fluent(
        fluent_path=fluent_path, dry_run=True, ui_mode="no_gui"
    )
    assert str(Path(server_info_file_name).parent) == tempfile.gettempdir()
    assert (
        fluent_launch_string
        == f"{fluent_path} 3ddp -gu -sifile={server_info_file_name} -nm"
    )


def test_standalone_launcher_dry_run_with_server_info_dir(monkeypatch):
    monkeypatch.setenv("PYFLUENT_LAUNCH_CONTAINER", "0")
    with tempfile.TemporaryDirectory() as tmp_dir:
        monkeypatch.setenv("SERVER_INFO_DIR", tmp_dir)
        fluent_path = r"\x\y\z\fluent.exe"
        fluent_launch_string, server_info_file_name = pyfluent.launch_fluent(
            fluent_path=fluent_path, dry_run=True, ui_mode="no_gui"
        )
        assert str(Path(server_info_file_name).parent) == tmp_dir
        assert (
            fluent_launch_string
            == f"{fluent_path} 3ddp -gu -sifile={Path(server_info_file_name).name} -nm"
        )


def test_container_ports():
    container_dict = {"ports": {"5000": 5000, "5001": 5001}}
    with pyfluent.launch_fluent(container_dict=container_dict) as session:
        session._container.reload()
        assert len(session._container.ports) == 2


def test_correct_ip_port():
    with pytest.raises(InvalidIpPort):
        pyfluent.connect_to_fluent(ip="1.2.3.4", port=5555)


def test_container_launcher_args():
    container_dict = pyfluent.launch_fluent(start_container=True, dry_run=True)
    commands = container_dict["command"]
    graphics_args = ["-gu", "-hidden", "-g", "-gr"]
    graphics_arg_count = 0
    for arg in graphics_args:
        graphics_arg_count += commands.count(arg)
    assert graphics_arg_count == 1, "Too many graphics arguments"


def test_report():
    from ansys.fluent.core.report import ANSYS_ENV_VARS, dependencies
    from ansys.tools.report import Report

    rep = Report(ansys_libs=dependencies, ansys_vars=ANSYS_ENV_VARS)
    assert "PyAnsys Software and Environment Report" in str(rep)
    assert str(rep).count("pandas") == 1



# Copyright (C) 2021 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Module controlling PyFluent's logging functionality."""

import logging.config
import os

_logging_file_enabled = False


def root_config():
    """Sets up the root PyFluent logger that outputs messages to stdout, but not to
    files."""
    logger = logging.getLogger("pyfluent")
    logger.setLevel("WARNING")
    formatter = logging.Formatter("%(name)s %(levelname)s: %(message)s")
    if not logger.handlers:
        ch = logging.StreamHandler()
        ch.setLevel("WARNING")
        ch.setFormatter(formatter)
        logger.addHandler(ch)


def set_console_logging_level(level: str | int):
    """Sets the level of PyFluent logging being output to console.

    Parameters
    ----------
    level : str or int
        Specified logging level to set PyFluent loggers to.

    Notes
    -----
    See logging levels in https://docs.python.org/3/library/logging.html#logging-levels
    """
    logger = logging.getLogger("pyfluent")
    logger.setLevel(level)
    for ch in logger.handlers:
        ch.setLevel(level)


def is_active() -> bool:
    """Returns whether PyFluent logging to file is active."""
    return _logging_file_enabled


def get_default_config() -> dict:
    """Returns the default configuration dictionary obtained from parsing from the
    PyFluent ``logging_config.yaml`` file.

    Examples
    --------
    >>> import ansys.fluent.core as pyfluent
    >>> pyfluent.logging.get_default_config()
    {'disable_existing_loggers': False,
     'formatters': {'logfile_fmt': {'format': '%(asctime)s %(name)-21s '
                                              '%(levelname)-8s %(message)s'}},
     'handlers': {'pyfluent_file': {'backupCount': 9,
                                    'class': 'logging.handlers.RotatingFileHandler',
                                    'filename': 'pyfluent.log',
                                    'formatter': 'logfile_fmt',
                                    'level': 'NOTSET',
                                    'maxBytes': 10485760}},
     'loggers': {'pyfluent.datamodel': {'handlers': ['pyfluent_file'],
                                        'level': 'DEBUG'},
                 'pyfluent.field_data': {'handlers': ['pyfluent_file'],
                                         'level': 'DEBUG'},
                 'pyfluent.general': {'handlers': ['pyfluent_file'],
                                      'level': 'DEBUG'},
                 'pyfluent.launcher': {'handlers': ['pyfluent_file'],
                                       'level': 'DEBUG'},
                 'pyfluent.networking': {'handlers': ['pyfluent_file'],
                                         'level': 'DEBUG'},
                 'pyfluent.post_objects': {'handlers': ['pyfluent_file'],
                                           'level': 'DEBUG'},
                 'pyfluent.settings_api': {'handlers': ['pyfluent_file'],
                                           'level': 'DEBUG'},
                 'pyfluent.tui': {'handlers': ['pyfluent_file'], 'level': 'DEBUG'}},
     'version': 1}
    """
    import yaml

    file_name = os.path.abspath(__file__)
    file_dir = os.path.dirname(file_name)
    yaml_path = os.path.join(file_dir, "logging_config.yaml")
    with open(yaml_path, "rt") as f:
        config = yaml.safe_load(f)
    return config


def enable(level: str | int = "DEBUG", custom_config: dict | None = None):
    """Enables PyFluent logging to file.

    Parameters
    ----------
    level : str or int, optional
        Specified logging level to set PyFluent loggers to. If omitted, level is set to DEBUG.
    custom_config : dict, optional
        Used to provide a customized logging configuration file that will be used instead
        of the ``logging_config.yaml`` file (see also :func:`get_default_config`).

    Notes
    -----
    See logging levels in https://docs.python.org/3/library/logging.html#logging-levels

    Examples
    --------
    Using the default logging setup:

    >>> import ansys.fluent.core as pyfluent
    >>> pyfluent.logging.enable()

    Customizing logging configuration (see also :func:`get_default_config`):

    >>> import ansys.fluent.core as pyfluent
    >>> config_dict = pyfluent.logging.get_default_config()
    >>> config_dict['handlers']['pyfluent_file']['filename'] = 'test.log'
    >>> pyfluent.logging.enable(custom_config=config_dict)
    """
    global _logging_file_enabled

    if _logging_file_enabled:
        print(
            "PyFluent logging to file is already active, overwriting previous configuration..."
        )

    _logging_file_enabled = True

    # Configure the logging system
    if custom_config is not None:
        config = custom_config
    else:
        config = get_default_config()

    logging.config.dictConfig(config)
    file_name = config["handlers"]["pyfluent_file"]["filename"]

    print(f"PyFluent logging file {os.path.join(os.getcwd(), file_name)}")

    set_global_level(level)


def get_logger(*args, **kwargs):
    """Retrieves logger.

    Convenience wrapper for Python's :func:`logging.getLogger` function.
    """
    return logging.getLogger(*args, **kwargs)


def set_global_level(level: str | int):
    """Changes the levels of all PyFluent loggers that write to log file.

    Parameters
    ----------
    level : str or int
        Specified logging level to set PyFluent loggers to.

    Notes
    -----
    See logging levels in https://docs.python.org/3/library/logging.html#logging-levels

    Examples
    --------
    >>> import ansys.fluent.core as pyfluent
    >>> pyfluent.logging.set_global_level(10)

    or

    >>> pyfluent.logging.set_global_level('DEBUG')
    """
    if not is_active():
        print("Logging is not active, enable it first.")
        return
    if isinstance(level, str):
        if level.isdigit():
            level = int(level)
        else:
            level = level.upper()
    print(f"Setting PyFluent global logging level to {level}.")
    pyfluent_loggers = list_loggers()
    for name in pyfluent_loggers:
        if name != "pyfluent":  # do not change the console root PyFluent logger
            logging.getLogger(name).setLevel(level)


def list_loggers():
    """List all PyFluent loggers.

    Returns
    -------
    list of str
        Each list element is a PyFluent logger name that can be individually controlled
        through :func:`ansys.fluent.core.logging.get_logger`.

    Notes
    -----
    PyFluent loggers use the standard Python logging library, for more details
    see https://docs.python.org/3/library/logging.html#logger-objects

    Examples
    --------
    >>> import ansys.fluent.core as pyfluent
    >>> pyfluent.logging.enable()
    >>> pyfluent.logging.list_loggers()
    ['pyfluent.general', 'pyfluent.launcher', 'pyfluent.networking', ...]
    >>> logger = pyfluent.logging.get_logger('pyfluent.networking')
    >>> logger
    <Logger pyfluent.networking (DEBUG)>
    >>> logger.setLevel('ERROR')
    >>> logger
    <Logger pyfluent.networking (ERROR)>
    """
    logger_dict = logging.root.manager.loggerDict
    pyfluent_loggers = []
    for name in logger_dict:
        if name.startswith("pyfluent"):
            pyfluent_loggers.append(name)
    return pyfluent_loggers


def configure_env_var() -> None:
    """Verifies whether ``PYFLUENT_LOGGING`` environment variable was defined in the
    system. Executed once automatically on PyFluent initialization.

    Notes
    -----
    The usual way to enable PyFluent logging to file is through :func:`enable()`.
    ``PYFLUENT_LOGGING`` set to ``0`` or ``OFF`` is the same as if no environment variable was set.
    If logging debug output to file by default is desired, without having to use :func:`enable()` every time,
    set environment variable ``PYFLUENT_LOGGING`` to ``DEBUG``.
    """
    env_logging_level = os.getenv("PYFLUENT_LOGGING")
    if env_logging_level:
        if env_logging_level.isdigit():
            env_logging_level = int(env_logging_level)
        else:
            env_logging_level = env_logging_level.upper()
        if not is_active() and env_logging_level not in [0, "OFF"]:
            print(
                "PYFLUENT_LOGGING environment variable specified, enabling logging..."
            )
            enable(env_logging_level)


root_config()
configure_env_var()



# Copyright (C) 2021 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Module providing dump session data functionality."""

from pathlib import Path
import pickle

import numpy as np

from ansys.fluent.core.services.field_data import SurfaceDataType


def dump_session_data(
    session,
    file_name: str,
    fields: list | None = None,
    surfaces: list | None = None,
):
    """Dump session data.

    Parameters
    ----------
    session :
        Session object.
    file_name: str
        File path for the session dump.
    fields: list, optional
        List of fields to write. If the list is empty, all fields are written.
    surfaces: list, optional
        List of surfaces to write. If the list is empty, all surfaces are written.
    """
    session_data = {
        "scalar_fields_info": {
            k: v
            for k, v in session.fields.field_info.get_scalar_fields_info().items()
            if (not fields or k in fields)
        },
        "surfaces_info": {
            k: v
            for k, v in session.fields.field_info.get_surfaces_info().items()
            if (not surfaces or k in surfaces)
        },
        "vector_fields_info": session.fields.field_info.get_vector_fields_info(),
    }
    if not fields:
        fields = [
            v["solver_name"] for k, v in session_data["scalar_fields_info"].items()
        ]
    surfaces_id = [v["surface_id"][0] for k, v in session_data["surfaces_info"].items()]
    session_data["range"] = {}
    for field in fields:
        session_data["range"][field] = {}
        for surface in surfaces_id:
            session_data["range"][field][surface] = {}
            session_data["range"][field][surface]["node_value"] = (
                session.fields.field_info.get_scalar_field_range(field, True, [surface])
            )
            session_data["range"][field][surface]["cell_value"] = (
                session.fields.field_info.get_scalar_field_range(
                    field, False, [surface]
                )
            )

    transaction = session.fields.field_data.new_transaction()
    transaction.add_surfaces_request(
        surface_ids=surfaces_id, provide_faces_centroid=True, provide_faces_normal=True
    )
    for field in fields:
        transaction.add_scalar_fields_request(
            surface_ids=surfaces_id,
            field_name=field,
            node_value=True,
            boundary_value=False,
        )
        transaction.add_scalar_fields_request(
            surface_ids=surfaces_id,
            field_name=field,
            node_value=False,
            boundary_value=False,
        )
        transaction.add_scalar_fields_request(
            surface_ids=surfaces_id,
            field_name=field,
            node_value=True,
            boundary_value=True,
        )
        transaction.add_scalar_fields_request(
            surface_ids=surfaces_id,
            field_name=field,
            node_value=False,
            boundary_value=True,
        )
    transaction.add_vector_fields_request(
        surface_ids=surfaces_id, field_name="velocity"
    )
    transaction.add_pathlines_fields_request(
        surface_ids=surfaces_id, field_name="velocity-magnitude"
    )
    session_data["fields"] = transaction.get_fields()

    with open(file_name, "wb") as pickle_obj:
        pickle.dump(session_data, pickle_obj)


class DumpDataReader:
    """Reads dump data."""

    def __init__(self, file_name: str):
        """Initialize DumpDataReader."""
        with open(
            str(Path(file_name).resolve()),
            "rb",
        ) as pickle_obj:
            self._session_data = pickle.load(pickle_obj)

    def get_session_data(self):
        """Get session data."""
        return self._session_data

    def get_surface_data(self, surface_ids, data_types) -> list[np.ndarray | None]:
        """Get surface data."""
        tag_id = (("type", "surface-data"),)

        surfaces_data = [
            self._session_data["fields"][tag_id][surface_id][
                SurfaceDataType(data_type).value
            ]
            for data_type in data_types
            for surface_id in surface_ids
        ]

        return surfaces_data

    def get_scalar_field_data(
        self, surface_ids, data_location, provide_boundary_values, field_names
    ) -> list[np.ndarray | None]:
        """Get scalar field data."""
        tag_id = (
            ("type", "scalar-field"),
            ("dataLocation", data_location),
            ("boundaryValues", provide_boundary_values),
        )

        scalar_field_data = [
            self._session_data["fields"][tag_id][surface_id][field_name]
            for field_name in field_names
            for surface_id in surface_ids
        ]

        return scalar_field_data

    def get_vector_field_data(
        self, surface_ids, field_names
    ) -> list[np.ndarray | None]:
        """Get vector field data."""
        tag_id = (("type", "vector-field"),)

        vector_field_data = [
            (
                self._session_data["fields"][tag_id][surface_id][field_name],
                self._session_data["fields"][tag_id][surface_id]["vector-scale"],
            )
            for field_name in field_names
            for surface_id in surface_ids
        ]

        return vector_field_data

    def get_pathlines_data(
        self, surface_ids, field_names, key
    ) -> list[np.ndarray | None]:
        """Get pathlines data."""
        pathlines_data = []
        for surface_id in surface_ids:
            for field_name in field_names:
                tag_id = (("type", "pathlines-field"), ("field", field_name))
                pathlines_data.append(
                    self._session_data["fields"][tag_id][surface_id][key]
                )

        return pathlines_data



"""Module to modify the HTML files generated by Sphinx."""

from importlib import import_module
from pathlib import Path

from bs4 import BeautifulSoup

from ansys.fluent.core.solver.flobject import Base


def modify_html(soup: BeautifulSoup) -> None:
    """Modify the HTML files generated by Sphinx to write original python class
    names."""
    module_name = soup.find(class_="sig-prename descclassname").text.removesuffix(".")
    module = import_module(module_name)
    class_desc = soup.find(class_="sig-name descname")
    class_name = class_desc.text
    cls = getattr(module, class_name)
    if issubclass(cls, Base):
        if cls.__name__ != cls._python_name:
            class_desc.string = cls._python_name


if __name__ == "__main__":
    html_dir = (
        Path(__file__).parent
        / "_build"
        / "html"
        / "api"
        / "solver"
        / "_autosummary"
        / "settings"
    )
    for html_file in html_dir.glob("*.html"):
        with open(html_file, "r", encoding="utf-8") as f:
            soup = BeautifulSoup(f, "html.parser", from_encoding="utf-8")

        modify_html(soup)

        with open(html_file, "w", encoding="utf-8") as f:
            f.write(str(soup))



# Copyright (C) 2021 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from ansys.fluent.core.generated.datamodel_251.preferences import (
    Root as preferences_root,
)
from ansys.fluent.core.generated.datamodel_251.workflow import Root as workflow_root
import ansys.fluent.core.generated.solver.settings_251 as settings_root
from ansys.fluent.core.generated.solver.tui_251 import main_menu
from ansys.fluent.core.system_coupling import SystemCoupling

class Solver:
    @property
    def version(self): ...
    @property
    def tui(self) -> main_menu: ...
    @property
    def workflow(self) -> workflow_root: ...
    @property
    def system_coupling(self) -> SystemCoupling: ...
    @property
    def preferences(self) -> preferences_root: ...
    def read_case_lightweight(self, file_name: str): ...
    def read_case(self, file_name: str): ...
    def write_case(self, file_name: str): ...
    @property
    def settings(self) -> settings_root.root: ...
